# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: wifi_service.proto
# pylint: skip-file
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gazoo_device.protos import common_pb2 as common__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='wifi_service.proto',
  package='chip.rpc',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x12wifi_service.proto\x12\x08\x63hip.rpc\x1a\x0c\x63ommon.proto\"\x1a\n\x07\x43hannel\x12\x0f\n\x07\x63hannel\x18\x01 \x01(\r\"\x14\n\x04Ssid\x12\x0c\n\x04ssid\x18\x01 \x01(\x0c\"\x1a\n\x05State\x12\x11\n\tconnected\x18\x01 \x01(\x08\"!\n\nMacAddress\x12\x13\n\x0bmac_address\x18\x01 \x01(\t\"\"\n\rWiFiInterface\x12\x11\n\tinterface\x18\x01 \x01(\t\"\x1d\n\nIP4Address\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\t\"\x1d\n\nIP6Address\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\t\"\x98\x01\n\nScanConfig\x12\x0c\n\x04ssid\x18\x01 \x03(\x0c\x12\r\n\x05\x62ssid\x18\x02 \x03(\x0c\x12\x0f\n\x07\x63hannel\x18\x03 \x01(\r\x12\x13\n\x0bshow_hidden\x18\x04 \x01(\x08\x12\x13\n\x0b\x61\x63tive_scan\x18\x05 \x01(\x08\x12\x18\n\x10scan_time_min_ms\x18\x06 \x01(\r\x12\x18\n\x10scan_time_max_ms\x18\x07 \x01(\r\"\x92\x01\n\nScanResult\x12\x0c\n\x04ssid\x18\x01 \x01(\x0c\x12\r\n\x05\x62ssid\x18\x02 \x01(\x0c\x12\x33\n\rsecurity_type\x18\x03 \x01(\x0e\x32\x1c.chip.rpc.WIFI_SECURITY_TYPE\x12\x11\n\tfrequency\x18\x04 \x01(\r\x12\x0f\n\x07\x63hannel\x18\x05 \x01(\r\x12\x0e\n\x06signal\x18\x06 \x01(\x05\"0\n\x0bScanResults\x12!\n\x03\x61ps\x18\x01 \x03(\x0b\x32\x14.chip.rpc.ScanResult\"c\n\x0e\x43onnectionData\x12\x0c\n\x04ssid\x18\x01 \x01(\x0c\x12\x33\n\rsecurity_type\x18\x02 \x01(\x0e\x32\x1c.chip.rpc.WIFI_SECURITY_TYPE\x12\x0e\n\x06secret\x18\x03 \x01(\x0c\"=\n\x10\x43onnectionResult\x12)\n\x05\x65rror\x18\x01 \x01(\x0e\x32\x1a.chip.rpc.CONNECTION_ERROR*\xf2\x01\n\x12WIFI_SECURITY_TYPE\x12\x12\n\x0eWIFI_AUTH_OPEN\x10\x00\x12\x11\n\rWIFI_AUTH_WEP\x10\x01\x12\x15\n\x11WIFI_AUTH_WPA_PSK\x10\x02\x12\x16\n\x12WIFI_AUTH_WPA2_PSK\x10\x03\x12\x1a\n\x16WIFI_AUTH_WPA_WPA2_PSK\x10\x04\x12\x1d\n\x19WIFI_AUTH_WPA2_ENTERPRISE\x10\x05\x12\x16\n\x12WIFI_AUTH_WPA3_PSK\x10\x06\x12\x1b\n\x17WIFI_AUTH_WPA2_WPA3_PSK\x10\x07\x12\x16\n\x12WIFI_AUTH_WAPI_PSK\x10\x08*\xbf\x05\n\x10\x43ONNECTION_ERROR\x12\x06\n\x02OK\x10\x00\x12\x0f\n\x0bUNSPECIFIED\x10\x01\x12\x0f\n\x0b\x41UTH_EXPIRE\x10\x02\x12\x0e\n\nAUTH_LEAVE\x10\x03\x12\x10\n\x0c\x41SSOC_EXPIRE\x10\x04\x12\x11\n\rASSOC_TOOMANY\x10\x05\x12\x0e\n\nNOT_AUTHED\x10\x06\x12\x0f\n\x0bNOT_ASSOCED\x10\x07\x12\x0f\n\x0b\x41SSOC_LEAVE\x10\x08\x12\x14\n\x10\x41SSOC_NOT_AUTHED\x10\t\x12\x17\n\x13\x44ISASSOC_PWRCAP_BAD\x10\n\x12\x18\n\x14\x44ISASSOC_SUPCHAN_BAD\x10\x0b\x12\x0e\n\nIE_INVALID\x10\r\x12\x0f\n\x0bMIC_FAILURE\x10\x0e\x12\x1d\n\x19\x46OURWAY_HANDSHAKE_TIMEOUT\x10\x0f\x12\x1c\n\x18GROUP_KEY_UPDATE_TIMEOUT\x10\x10\x12\x16\n\x12IE_IN_4WAY_DIFFERS\x10\x11\x12\x18\n\x14GROUP_CIPHER_INVALID\x10\x12\x12\x1b\n\x17PAIRWISE_CIPHER_INVALID\x10\x13\x12\x10\n\x0c\x41KMP_INVALID\x10\x14\x12\x19\n\x15UNSUPP_RSN_IE_VERSION\x10\x15\x12\x16\n\x12INVALID_RSN_IE_CAP\x10\x16\x12\x1a\n\x16IEEE802_1X_AUTH_FAILED\x10\x17\x12\x19\n\x15\x43IPHER_SUITE_REJECTED\x10\x18\x12\x11\n\rINVALID_PMKID\x10\x35\x12\x13\n\x0e\x42\x45\x41\x43ON_TIMEOUT\x10\xc8\x01\x12\x10\n\x0bNO_AP_FOUND\x10\xc9\x01\x12\x0e\n\tAUTH_FAIL\x10\xca\x01\x12\x0f\n\nASSOC_FAIL\x10\xcb\x01\x12\x16\n\x11HANDSHAKE_TIMEOUT\x10\xcc\x01\x12\x14\n\x0f\x43ONNECTION_FAIL\x10\xcd\x01\x12\x11\n\x0c\x41P_TSF_RESET\x10\xce\x01\x12\x0c\n\x07ROAMING\x10\xcf\x01\x32\x8a\x05\n\x04WiFi\x12\x35\n\nGetChannel\x12\x12.pw.protobuf.Empty\x1a\x11.chip.rpc.Channel\"\x00\x12/\n\x07GetSsid\x12\x12.pw.protobuf.Empty\x1a\x0e.chip.rpc.Ssid\"\x00\x12\x31\n\x08GetState\x12\x12.pw.protobuf.Empty\x1a\x0f.chip.rpc.State\"\x00\x12;\n\rGetMacAddress\x12\x12.pw.protobuf.Empty\x1a\x14.chip.rpc.MacAddress\"\x00\x12\x41\n\x10GetWiFiInterface\x12\x12.pw.protobuf.Empty\x1a\x17.chip.rpc.WiFiInterface\"\x00\x12;\n\rGetIP4Address\x12\x12.pw.protobuf.Empty\x1a\x14.chip.rpc.IP4Address\"\x00\x12;\n\rGetIP6Address\x12\x12.pw.protobuf.Empty\x1a\x14.chip.rpc.IP6Address\"\x00\x12<\n\tStartScan\x12\x14.chip.rpc.ScanConfig\x1a\x15.chip.rpc.ScanResults\"\x00\x30\x01\x12\x34\n\x08StopScan\x12\x12.pw.protobuf.Empty\x1a\x12.pw.protobuf.Empty\"\x00\x12\x41\n\x07\x43onnect\x12\x18.chip.rpc.ConnectionData\x1a\x1a.chip.rpc.ConnectionResult\"\x00\x12\x36\n\nDisconnect\x12\x12.pw.protobuf.Empty\x1a\x12.pw.protobuf.Empty\"\x00\x62\x06proto3'
  ,
  dependencies=[common__pb2.DESCRIPTOR,])

_WIFI_SECURITY_TYPE = _descriptor.EnumDescriptor(
  name='WIFI_SECURITY_TYPE',
  full_name='chip.rpc.WIFI_SECURITY_TYPE',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='WIFI_AUTH_OPEN', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='WIFI_AUTH_WEP', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='WIFI_AUTH_WPA_PSK', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='WIFI_AUTH_WPA2_PSK', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='WIFI_AUTH_WPA_WPA2_PSK', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='WIFI_AUTH_WPA2_ENTERPRISE', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='WIFI_AUTH_WPA3_PSK', index=6, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='WIFI_AUTH_WPA2_WPA3_PSK', index=7, number=7,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='WIFI_AUTH_WAPI_PSK', index=8, number=8,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=776,
  serialized_end=1018,
)
_sym_db.RegisterEnumDescriptor(_WIFI_SECURITY_TYPE)

WIFI_SECURITY_TYPE = enum_type_wrapper.EnumTypeWrapper(_WIFI_SECURITY_TYPE)
_CONNECTION_ERROR = _descriptor.EnumDescriptor(
  name='CONNECTION_ERROR',
  full_name='chip.rpc.CONNECTION_ERROR',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='OK', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='UNSPECIFIED', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='AUTH_EXPIRE', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='AUTH_LEAVE', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ASSOC_EXPIRE', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ASSOC_TOOMANY', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='NOT_AUTHED', index=6, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='NOT_ASSOCED', index=7, number=7,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ASSOC_LEAVE', index=8, number=8,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ASSOC_NOT_AUTHED', index=9, number=9,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DISASSOC_PWRCAP_BAD', index=10, number=10,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DISASSOC_SUPCHAN_BAD', index=11, number=11,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='IE_INVALID', index=12, number=13,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MIC_FAILURE', index=13, number=14,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='FOURWAY_HANDSHAKE_TIMEOUT', index=14, number=15,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='GROUP_KEY_UPDATE_TIMEOUT', index=15, number=16,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='IE_IN_4WAY_DIFFERS', index=16, number=17,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='GROUP_CIPHER_INVALID', index=17, number=18,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PAIRWISE_CIPHER_INVALID', index=18, number=19,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='AKMP_INVALID', index=19, number=20,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='UNSUPP_RSN_IE_VERSION', index=20, number=21,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='INVALID_RSN_IE_CAP', index=21, number=22,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='IEEE802_1X_AUTH_FAILED', index=22, number=23,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CIPHER_SUITE_REJECTED', index=23, number=24,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='INVALID_PMKID', index=24, number=53,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='BEACON_TIMEOUT', index=25, number=200,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='NO_AP_FOUND', index=26, number=201,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='AUTH_FAIL', index=27, number=202,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ASSOC_FAIL', index=28, number=203,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='HANDSHAKE_TIMEOUT', index=29, number=204,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CONNECTION_FAIL', index=30, number=205,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='AP_TSF_RESET', index=31, number=206,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ROAMING', index=32, number=207,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1021,
  serialized_end=1724,
)
_sym_db.RegisterEnumDescriptor(_CONNECTION_ERROR)

CONNECTION_ERROR = enum_type_wrapper.EnumTypeWrapper(_CONNECTION_ERROR)
WIFI_AUTH_OPEN = 0
WIFI_AUTH_WEP = 1
WIFI_AUTH_WPA_PSK = 2
WIFI_AUTH_WPA2_PSK = 3
WIFI_AUTH_WPA_WPA2_PSK = 4
WIFI_AUTH_WPA2_ENTERPRISE = 5
WIFI_AUTH_WPA3_PSK = 6
WIFI_AUTH_WPA2_WPA3_PSK = 7
WIFI_AUTH_WAPI_PSK = 8
OK = 0
UNSPECIFIED = 1
AUTH_EXPIRE = 2
AUTH_LEAVE = 3
ASSOC_EXPIRE = 4
ASSOC_TOOMANY = 5
NOT_AUTHED = 6
NOT_ASSOCED = 7
ASSOC_LEAVE = 8
ASSOC_NOT_AUTHED = 9
DISASSOC_PWRCAP_BAD = 10
DISASSOC_SUPCHAN_BAD = 11
IE_INVALID = 13
MIC_FAILURE = 14
FOURWAY_HANDSHAKE_TIMEOUT = 15
GROUP_KEY_UPDATE_TIMEOUT = 16
IE_IN_4WAY_DIFFERS = 17
GROUP_CIPHER_INVALID = 18
PAIRWISE_CIPHER_INVALID = 19
AKMP_INVALID = 20
UNSUPP_RSN_IE_VERSION = 21
INVALID_RSN_IE_CAP = 22
IEEE802_1X_AUTH_FAILED = 23
CIPHER_SUITE_REJECTED = 24
INVALID_PMKID = 53
BEACON_TIMEOUT = 200
NO_AP_FOUND = 201
AUTH_FAIL = 202
ASSOC_FAIL = 203
HANDSHAKE_TIMEOUT = 204
CONNECTION_FAIL = 205
AP_TSF_RESET = 206
ROAMING = 207



_CHANNEL = _descriptor.Descriptor(
  name='Channel',
  full_name='chip.rpc.Channel',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='channel', full_name='chip.rpc.Channel.channel', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=46,
  serialized_end=72,
)


_SSID = _descriptor.Descriptor(
  name='Ssid',
  full_name='chip.rpc.Ssid',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='ssid', full_name='chip.rpc.Ssid.ssid', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=74,
  serialized_end=94,
)


_STATE = _descriptor.Descriptor(
  name='State',
  full_name='chip.rpc.State',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='connected', full_name='chip.rpc.State.connected', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=96,
  serialized_end=122,
)


_MACADDRESS = _descriptor.Descriptor(
  name='MacAddress',
  full_name='chip.rpc.MacAddress',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='mac_address', full_name='chip.rpc.MacAddress.mac_address', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=124,
  serialized_end=157,
)


_WIFIINTERFACE = _descriptor.Descriptor(
  name='WiFiInterface',
  full_name='chip.rpc.WiFiInterface',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='interface', full_name='chip.rpc.WiFiInterface.interface', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=159,
  serialized_end=193,
)


_IP4ADDRESS = _descriptor.Descriptor(
  name='IP4Address',
  full_name='chip.rpc.IP4Address',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='address', full_name='chip.rpc.IP4Address.address', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=195,
  serialized_end=224,
)


_IP6ADDRESS = _descriptor.Descriptor(
  name='IP6Address',
  full_name='chip.rpc.IP6Address',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='address', full_name='chip.rpc.IP6Address.address', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=226,
  serialized_end=255,
)


_SCANCONFIG = _descriptor.Descriptor(
  name='ScanConfig',
  full_name='chip.rpc.ScanConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='ssid', full_name='chip.rpc.ScanConfig.ssid', index=0,
      number=1, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bssid', full_name='chip.rpc.ScanConfig.bssid', index=1,
      number=2, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='channel', full_name='chip.rpc.ScanConfig.channel', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='show_hidden', full_name='chip.rpc.ScanConfig.show_hidden', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='active_scan', full_name='chip.rpc.ScanConfig.active_scan', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='scan_time_min_ms', full_name='chip.rpc.ScanConfig.scan_time_min_ms', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='scan_time_max_ms', full_name='chip.rpc.ScanConfig.scan_time_max_ms', index=6,
      number=7, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=258,
  serialized_end=410,
)


_SCANRESULT = _descriptor.Descriptor(
  name='ScanResult',
  full_name='chip.rpc.ScanResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='ssid', full_name='chip.rpc.ScanResult.ssid', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bssid', full_name='chip.rpc.ScanResult.bssid', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='security_type', full_name='chip.rpc.ScanResult.security_type', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='frequency', full_name='chip.rpc.ScanResult.frequency', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='channel', full_name='chip.rpc.ScanResult.channel', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='signal', full_name='chip.rpc.ScanResult.signal', index=5,
      number=6, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=413,
  serialized_end=559,
)


_SCANRESULTS = _descriptor.Descriptor(
  name='ScanResults',
  full_name='chip.rpc.ScanResults',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='aps', full_name='chip.rpc.ScanResults.aps', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=561,
  serialized_end=609,
)


_CONNECTIONDATA = _descriptor.Descriptor(
  name='ConnectionData',
  full_name='chip.rpc.ConnectionData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='ssid', full_name='chip.rpc.ConnectionData.ssid', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='security_type', full_name='chip.rpc.ConnectionData.security_type', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='secret', full_name='chip.rpc.ConnectionData.secret', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=611,
  serialized_end=710,
)


_CONNECTIONRESULT = _descriptor.Descriptor(
  name='ConnectionResult',
  full_name='chip.rpc.ConnectionResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='error', full_name='chip.rpc.ConnectionResult.error', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=712,
  serialized_end=773,
)

_SCANRESULT.fields_by_name['security_type'].enum_type = _WIFI_SECURITY_TYPE
_SCANRESULTS.fields_by_name['aps'].message_type = _SCANRESULT
_CONNECTIONDATA.fields_by_name['security_type'].enum_type = _WIFI_SECURITY_TYPE
_CONNECTIONRESULT.fields_by_name['error'].enum_type = _CONNECTION_ERROR
DESCRIPTOR.message_types_by_name['Channel'] = _CHANNEL
DESCRIPTOR.message_types_by_name['Ssid'] = _SSID
DESCRIPTOR.message_types_by_name['State'] = _STATE
DESCRIPTOR.message_types_by_name['MacAddress'] = _MACADDRESS
DESCRIPTOR.message_types_by_name['WiFiInterface'] = _WIFIINTERFACE
DESCRIPTOR.message_types_by_name['IP4Address'] = _IP4ADDRESS
DESCRIPTOR.message_types_by_name['IP6Address'] = _IP6ADDRESS
DESCRIPTOR.message_types_by_name['ScanConfig'] = _SCANCONFIG
DESCRIPTOR.message_types_by_name['ScanResult'] = _SCANRESULT
DESCRIPTOR.message_types_by_name['ScanResults'] = _SCANRESULTS
DESCRIPTOR.message_types_by_name['ConnectionData'] = _CONNECTIONDATA
DESCRIPTOR.message_types_by_name['ConnectionResult'] = _CONNECTIONRESULT
DESCRIPTOR.enum_types_by_name['WIFI_SECURITY_TYPE'] = _WIFI_SECURITY_TYPE
DESCRIPTOR.enum_types_by_name['CONNECTION_ERROR'] = _CONNECTION_ERROR
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Channel = _reflection.GeneratedProtocolMessageType('Channel', (_message.Message,), {
  'DESCRIPTOR' : _CHANNEL,
  '__module__' : 'wifi_service_pb2'
  # @@protoc_insertion_point(class_scope:chip.rpc.Channel)
  })
_sym_db.RegisterMessage(Channel)

Ssid = _reflection.GeneratedProtocolMessageType('Ssid', (_message.Message,), {
  'DESCRIPTOR' : _SSID,
  '__module__' : 'wifi_service_pb2'
  # @@protoc_insertion_point(class_scope:chip.rpc.Ssid)
  })
_sym_db.RegisterMessage(Ssid)

State = _reflection.GeneratedProtocolMessageType('State', (_message.Message,), {
  'DESCRIPTOR' : _STATE,
  '__module__' : 'wifi_service_pb2'
  # @@protoc_insertion_point(class_scope:chip.rpc.State)
  })
_sym_db.RegisterMessage(State)

MacAddress = _reflection.GeneratedProtocolMessageType('MacAddress', (_message.Message,), {
  'DESCRIPTOR' : _MACADDRESS,
  '__module__' : 'wifi_service_pb2'
  # @@protoc_insertion_point(class_scope:chip.rpc.MacAddress)
  })
_sym_db.RegisterMessage(MacAddress)

WiFiInterface = _reflection.GeneratedProtocolMessageType('WiFiInterface', (_message.Message,), {
  'DESCRIPTOR' : _WIFIINTERFACE,
  '__module__' : 'wifi_service_pb2'
  # @@protoc_insertion_point(class_scope:chip.rpc.WiFiInterface)
  })
_sym_db.RegisterMessage(WiFiInterface)

IP4Address = _reflection.GeneratedProtocolMessageType('IP4Address', (_message.Message,), {
  'DESCRIPTOR' : _IP4ADDRESS,
  '__module__' : 'wifi_service_pb2'
  # @@protoc_insertion_point(class_scope:chip.rpc.IP4Address)
  })
_sym_db.RegisterMessage(IP4Address)

IP6Address = _reflection.GeneratedProtocolMessageType('IP6Address', (_message.Message,), {
  'DESCRIPTOR' : _IP6ADDRESS,
  '__module__' : 'wifi_service_pb2'
  # @@protoc_insertion_point(class_scope:chip.rpc.IP6Address)
  })
_sym_db.RegisterMessage(IP6Address)

ScanConfig = _reflection.GeneratedProtocolMessageType('ScanConfig', (_message.Message,), {
  'DESCRIPTOR' : _SCANCONFIG,
  '__module__' : 'wifi_service_pb2'
  # @@protoc_insertion_point(class_scope:chip.rpc.ScanConfig)
  })
_sym_db.RegisterMessage(ScanConfig)

ScanResult = _reflection.GeneratedProtocolMessageType('ScanResult', (_message.Message,), {
  'DESCRIPTOR' : _SCANRESULT,
  '__module__' : 'wifi_service_pb2'
  # @@protoc_insertion_point(class_scope:chip.rpc.ScanResult)
  })
_sym_db.RegisterMessage(ScanResult)

ScanResults = _reflection.GeneratedProtocolMessageType('ScanResults', (_message.Message,), {
  'DESCRIPTOR' : _SCANRESULTS,
  '__module__' : 'wifi_service_pb2'
  # @@protoc_insertion_point(class_scope:chip.rpc.ScanResults)
  })
_sym_db.RegisterMessage(ScanResults)

ConnectionData = _reflection.GeneratedProtocolMessageType('ConnectionData', (_message.Message,), {
  'DESCRIPTOR' : _CONNECTIONDATA,
  '__module__' : 'wifi_service_pb2'
  # @@protoc_insertion_point(class_scope:chip.rpc.ConnectionData)
  })
_sym_db.RegisterMessage(ConnectionData)

ConnectionResult = _reflection.GeneratedProtocolMessageType('ConnectionResult', (_message.Message,), {
  'DESCRIPTOR' : _CONNECTIONRESULT,
  '__module__' : 'wifi_service_pb2'
  # @@protoc_insertion_point(class_scope:chip.rpc.ConnectionResult)
  })
_sym_db.RegisterMessage(ConnectionResult)



_WIFI = _descriptor.ServiceDescriptor(
  name='WiFi',
  full_name='chip.rpc.WiFi',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=1727,
  serialized_end=2377,
  methods=[
  _descriptor.MethodDescriptor(
    name='GetChannel',
    full_name='chip.rpc.WiFi.GetChannel',
    index=0,
    containing_service=None,
    input_type=common__pb2._EMPTY,
    output_type=_CHANNEL,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetSsid',
    full_name='chip.rpc.WiFi.GetSsid',
    index=1,
    containing_service=None,
    input_type=common__pb2._EMPTY,
    output_type=_SSID,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetState',
    full_name='chip.rpc.WiFi.GetState',
    index=2,
    containing_service=None,
    input_type=common__pb2._EMPTY,
    output_type=_STATE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetMacAddress',
    full_name='chip.rpc.WiFi.GetMacAddress',
    index=3,
    containing_service=None,
    input_type=common__pb2._EMPTY,
    output_type=_MACADDRESS,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetWiFiInterface',
    full_name='chip.rpc.WiFi.GetWiFiInterface',
    index=4,
    containing_service=None,
    input_type=common__pb2._EMPTY,
    output_type=_WIFIINTERFACE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetIP4Address',
    full_name='chip.rpc.WiFi.GetIP4Address',
    index=5,
    containing_service=None,
    input_type=common__pb2._EMPTY,
    output_type=_IP4ADDRESS,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetIP6Address',
    full_name='chip.rpc.WiFi.GetIP6Address',
    index=6,
    containing_service=None,
    input_type=common__pb2._EMPTY,
    output_type=_IP6ADDRESS,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='StartScan',
    full_name='chip.rpc.WiFi.StartScan',
    index=7,
    containing_service=None,
    input_type=_SCANCONFIG,
    output_type=_SCANRESULTS,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='StopScan',
    full_name='chip.rpc.WiFi.StopScan',
    index=8,
    containing_service=None,
    input_type=common__pb2._EMPTY,
    output_type=common__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='Connect',
    full_name='chip.rpc.WiFi.Connect',
    index=9,
    containing_service=None,
    input_type=_CONNECTIONDATA,
    output_type=_CONNECTIONRESULT,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='Disconnect',
    full_name='chip.rpc.WiFi.Disconnect',
    index=10,
    containing_service=None,
    input_type=common__pb2._EMPTY,
    output_type=common__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_WIFI)

DESCRIPTOR.services_by_name['WiFi'] = _WIFI

# @@protoc_insertion_point(module_scope)



# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Device controller for NRF Matter device.

This device controller populates the supported Matter endpoints on the NRF
platform by using the descriptor RPC service.
"""
from gazoo_device import decorators
from gazoo_device import detect_criteria
from gazoo_device import gdm_logger
from gazoo_device.base_classes import matter_device_base
from gazoo_device.capabilities import flash_build_jlink

logger = gdm_logger.get_logger()
_NRF_JLINK_NAME = "NRF52840_XXAA"


class NrfMatter(matter_device_base.MatterDeviceBase):
  """NRF Matter device controller."""
  DETECT_MATCH_CRITERIA = {
      detect_criteria.PigweedQuery.IS_MATTER: True,
      detect_criteria.PigweedQuery.MANUFACTURER_NAME: "segger",
      detect_criteria.PigweedQuery.PRODUCT_NAME: "j-link",
  }
  # Button definition on Nordic dev board:
  # https://github.com/project-chip/connectedhomeip/tree/master/examples/lighting-app/nrfconnect#device-ui
  VALID_BUTTON_IDS = (0, 1, 2, 3)

  DEVICE_TYPE = "nrfmatter"

  @decorators.PersistentProperty
  def os(self) -> str:
    """OS of NRF platform."""
    return "Zephyr RTOS"

  @decorators.PersistentProperty
  def platform(self) -> str:
    """NRF platform."""
    return "nRF Connect"

  @decorators.CapabilityDecorator(flash_build_jlink.FlashBuildJLink)
  def flash_build(self) -> flash_build_jlink.FlashBuildJLink:
    """FlashBuildJLink capability to flash hex image."""
    return self.lazy_init(flash_build_jlink.FlashBuildJLink,
                          device_name=self.name,
                          serial_number=self.serial_number,
                          platform_name=_NRF_JLINK_NAME,
                          reset_endpoints_fn=self.matter_endpoints.reset,
                          switchboard=self.switchboard)



# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Interface for a PwRPC (Pigweed RPC) wifi capability."""
import abc
from typing import Optional

from gazoo_device.capabilities.interfaces import capability_base


class PwRPCWifiBase(capability_base.CapabilityBase):
  """Pigweed RPC Wi-Fi capability."""

  @property
  @abc.abstractmethod
  def channel(self) -> int:
    """Wifi channel used with current ssid connection."""

  @property
  @abc.abstractmethod
  def ssid(self) -> str:
    """Name of the SSID to which device is connected."""

  @property
  @abc.abstractmethod
  def mac_address(self) -> str:
    """MAC address for the wifi interface."""

  @property
  @abc.abstractmethod
  def wifi_interface(self) -> str:
    """Name of the interface used for wifi connection."""

  @property
  @abc.abstractmethod
  def ipv4_address(self) -> str:
    """IPv4 address for wifi interface."""

  @abc.abstractmethod
  def connect(self,
              ssid: str,
              security_type: str,
              secret: Optional[str] = None) -> None:
    """Establishes wifi conection with given ssid.

    Args:
      ssid: Name of the wifi ssid.
      security_type: Wifi encryption protocol as defined in
        wifi_service_pb2.WIFI_SECURITY_TYPE.
      secret: Wifi passphrase.

    Raises:
      DeviceError with appropriate error code if connection failed.
    """

  @abc.abstractmethod
  def disconnect(self) -> None:
    """Disconnects from current wifi connection."""



# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Base class module for Silabs EFR32 platform device."""
import os
from typing import Dict, Tuple

from gazoo_device import console_config
from gazoo_device import custom_types
from gazoo_device import decorators
from gazoo_device import gdm_logger
from gazoo_device.base_classes import auxiliary_device
from gazoo_device.capabilities import flash_build_jlink
from gazoo_device.utility import usb_utils

logger = gdm_logger.get_logger()
BAUDRATE = 115200
_EFR32_JLINK_NAME = "EFR32MG12PXXXF1024"


class SilabsEFR32Device(auxiliary_device.AuxiliaryDevice):
  """Base class for Silabs EFR32MG platform device.

  EFR32MG devices from Silabs which include Zigbee and Thread software stacks.
  """
  COMMUNICATION_TYPE = "PigweedSerialComms"
  _COMMUNICATION_KWARGS = {"protobufs": None, "baudrate": BAUDRATE}

  def get_console_configuration(self) -> console_config.ConsoleConfiguration:
    """Returns the interactive console configuration."""
    return console_config.get_log_only_configuration()

  @decorators.LogDecorator(logger)
  def get_detection_info(self) -> Tuple[Dict[str, str], Dict[str, str]]:
    """Gets the persistent and optional attributes of a device during setup.

    Returns:
      Dictionary of persistent attributes and dictionary of
      optional attributes.
    """
    persistent_dict = self.props["persistent_identifiers"]
    address = persistent_dict["console_port_name"]
    persistent_dict["serial_number"] = (
        usb_utils.get_serial_number_from_path(address))
    persistent_dict["model"] = "PROTO"
    return persistent_dict, {}

  @classmethod
  def is_connected(cls,
                   device_config: custom_types.ManagerDeviceConfigDict) -> bool:
    """Returns True if the device is connected to the host."""
    return os.path.exists(device_config["persistent"]["console_port_name"])

  @decorators.PersistentProperty
  def os(self) -> str:
    return "FreeRTOS"

  @decorators.PersistentProperty
  def platform(self) -> str:
    return "EFR32MG"

  @decorators.CapabilityDecorator(flash_build_jlink.FlashBuildJLink)
  def flash_build(self):
    return self.lazy_init(flash_build_jlink.FlashBuildJLink,
                          device_name=self.name,
                          serial_number=self.serial_number,
                          platform_name=_EFR32_JLINK_NAME)



# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Queries sent to devices during detection to determine their device type."""
import copy
import enum
import functools
import logging
import re
import subprocess
import typing
from typing import Any, Callable, Collection, Dict, List, Union

from gazoo_device import config
from gazoo_device import errors
from gazoo_device import extensions
from gazoo_device.base_classes import auxiliary_device_base
from gazoo_device.base_classes import primary_device_base
from gazoo_device.capabilities.interfaces import switchboard_base
from gazoo_device.utility import host_utils
from gazoo_device.utility import http_utils
from gazoo_device.utility import pwrpc_utils
from gazoo_device.utility import usb_utils
import immutabledict
import requests

_DeviceClassType = Union[auxiliary_device_base.AuxiliaryDeviceBase,
                         primary_device_base.PrimaryDeviceBase]
_LOG_FORMAT = "<%(asctime)s> GDM-M: %(message)s"

_ADB_COMMANDS = immutabledict.immutabledict({})

_DOCKER_COMMANDS = immutabledict.immutabledict({
    "PRODUCT_NAME": "docker ps --filter id={} --format {{{{.Names}}}}",
})

_SSH_COMMANDS = immutabledict.immutabledict({
    "IS_CHIP_TOOL_PRESENT": "which chip-tool",
    "UNIFI_PRODUCT_NAME": "mca-cli-op info",
    "DLI_PRODUCT_NAME": "http://{address}/restapi/config/=brand_name/",
    "RPI_PRODUCT_NAME": "cat /proc/device-tree/model",
})

_GET_DLINK_MODEL_SNMP_COMMAND = (
    "snmpget -v 2c -c private {ip_address}:161 1.3.6.1.2.1.1.1.0")
_MODEL_RESPONSE_REG_EX = r"DGS-1100.+? Gigabit Ethernet Switch"
_SNMP_TIMEOUT_S = 10

_UNIFI_MODEL_PREFIXES = ("USW-", "US-")


@functools.total_ordering
class QueryEnum(enum.Enum):
  """Allows comparison of enum properties for sorting purposes."""

  def __lt__(self, other):
    return self.name < other.name  # pylint: disable=comparison-with-callable


class AdbQuery(QueryEnum):
  IS_SERIAL_NUMBER = "is_serial_number"


class DockerQuery(QueryEnum):
  PRODUCT_NAME = "product_name"


class GenericQuery(QueryEnum):
  ALWAYS_TRUE = "always_true"


class PigweedQuery(QueryEnum):
  """Query names for detection for PigweedSerialComms Devices."""
  IS_MATTER = "is_matter"
  PRODUCT_NAME = "usb info product_name"
  MANUFACTURER_NAME = "usb info manufacturer_name"


class PtyProcessQuery(QueryEnum):
  PRODUCT_NAME = "product_name"


class SerialQuery(QueryEnum):
  PRODUCT_NAME = "usb info product_name"
  SERIAL_NUMBER = "usb serial_numer"
  VENDOR_PRODUCT_ID = "VENDOR_ID:PRODUCT_ID"


class SnmpQuery(QueryEnum):
  """Query names for detection of SnmpComms Devices."""
  IS_DLINK = "is_dlink_power_switch"


class SshQuery(QueryEnum):
  """Query names for detection for SshComms Devices."""
  IS_DLI = "is_dli_power_switch"
  IS_RPI = "is_raspberry_pi"
  IS_UNIFI = "is_unifi_switch"
  IS_CHIP_TOOL_PRESENT = "is_chip_tool_installed_on_rpi"


class UsbQuery(QueryEnum):
  PRODUCT_NAME = "usb product name"
  SERIAL_NUMBER = "serial_number"
  VENDOR_PRODUCT_ID = "VENDOR_ID:PRODUCT_ID"


def _docker_product_name_query(
    address: str, detect_logger: logging.Logger,
    create_switchboard_func: Callable[..., switchboard_base.SwitchboardBase]
) -> str:
  """Gets product name from docker device."""
  del create_switchboard_func  # Unused by _docker_product_name_query
  try:
    name = subprocess.check_output(
        _DOCKER_COMMANDS["PRODUCT_NAME"].format(address).split())
    name = name.decode()
  except subprocess.CalledProcessError as err:
    detect_logger.info(
        "_docker_product_name_query failed for %s: %r", address, err)
    return ""
  return name


def _always_true_query(
    address: str, detect_logger: logging.Logger,
    create_switchboard_func: Callable[..., switchboard_base.SwitchboardBase]
) -> bool:
  """Used when there is just one type of device for a communication type."""
  del address, create_switchboard_func, detect_logger  # Unused.
  return True


def _adb_is_serial_number_query(
    address: str,
    detect_logger: logging.Logger,
    create_switchboard_func: Callable[..., switchboard_base.SwitchboardBase]
) -> bool:
  """Returns True if address is a serial number, False if it's an IP address."""
  del create_switchboard_func, detect_logger  # Unused.
  return not re.search(host_utils.IP_ADDRESS, address)


def _is_dli_query(
    address: str, detect_logger: logging.Logger,
    create_switchboard_func: Callable[..., switchboard_base.SwitchboardBase]
) -> bool:
  """Determines if address belongs to dli power switch."""
  del create_switchboard_func  # Unused by _is_dli_query
  try:
    response = http_utils.send_http_get(
        _SSH_COMMANDS["DLI_PRODUCT_NAME"].format(address=address),
        auth=requests.auth.HTTPDigestAuth("admin", "1234"),
        headers={"Accept": "application/json"},
        valid_return_codes=[200, 206, 207],
        timeout=1)
    name = response.text
  except RuntimeError as err:
    detect_logger.info("_is_dli_query failed for %s: %r", address, err)
    return False
  return "Power Switch" in name


def get_dlink_model_name(ip_address: str) -> str:
  """Returns the model name of the Dlink switch at the ip_address."""
  command = _GET_DLINK_MODEL_SNMP_COMMAND.format(ip_address=ip_address)
  # Expected response for 5 port model should look like:
  # "DGS-1100-05 Gigabit Ethernet Switch"
  response = subprocess.check_output(
      command.split(), text=True, timeout=_SNMP_TIMEOUT_S)
  match = re.search(_MODEL_RESPONSE_REG_EX, response)
  if match:
    return match[0]
  raise errors.DeviceError(f"Failed to retrieve model name from dlink_switch "
                           f"with command: {command}\n"
                           f"Unexpected output: {response}")


def _is_dlink_query(
    address: str, detect_logger: logging.Logger,
    create_switchboard_func: Callable[..., switchboard_base.SwitchboardBase]
) -> bool:
  """Determines if address belongs to dli power switch."""
  del create_switchboard_func  # Unused by _is_dlink_query
  try:
    model = get_dlink_model_name(ip_address=address)
    detect_logger.info(f"Got response to sysDescr SNMP query: {model}\n"
                       f"_is_dlink_query response: True")
  except (subprocess.CalledProcessError, errors.DeviceError) as err:
    detect_logger.info("_is_dlink_query failure: " + repr(err))
    return False
  return True


def _is_rpi_query(
    address: str, detect_logger: logging.Logger,
    create_switchboard_func: Callable[..., switchboard_base.SwitchboardBase]
) -> bool:
  """Determines if address belongs to raspberry pi."""
  del create_switchboard_func  # Unused by _is_rpi_query
  try:
    name = host_utils.ssh_command(
        address,
        command=_SSH_COMMANDS["RPI_PRODUCT_NAME"],
        user="pi",
        key_info=config.KEYS["raspberrypi3_ssh_key"])
  except RuntimeError as err:
    detect_logger.info("_is_rpi_query failed for %s: %r", address, err)
    return False
  return "Raspberry Pi" in name


def _is_chip_tool_installed_query(
    address: str, detect_logger: logging.Logger,
    create_switchboard_func: Callable[..., switchboard_base.SwitchboardBase]
) -> bool:
  """Determines whether chip-tool is installed on an RPI."""
  del create_switchboard_func  # Unused by _is_chip_tool_installed_query
  try:
    host_utils.ssh_command(
        address,
        command=_SSH_COMMANDS["IS_CHIP_TOOL_PRESENT"],
        user="pi",
        key_info=config.KEYS["raspberrypi3_ssh_key"])
  except RuntimeError as err:
    detect_logger.info(
        "_is_chip_tool_installed_query failed for %s: %r", address, err)
    return False
  return True


def _is_unifi_query(
    address: str, detect_logger: logging.Logger,
    create_switchboard_func: Callable[..., switchboard_base.SwitchboardBase]
) -> bool:
  """Determines if address belongs to unifi poe switch."""
  del create_switchboard_func  # Unused by _is_unifi_query
  try:
    mca_info = host_utils.ssh_command(
        address,
        _SSH_COMMANDS["UNIFI_PRODUCT_NAME"],
        user="admin",
        key_info=config.KEYS["unifi_switch_ssh_key"])
  except RuntimeError as err:
    detect_logger.info("_is_unifi_query failed for %s: %r", address, err)
    return False

  for model_prefix in _UNIFI_MODEL_PREFIXES:
    if model_prefix in mca_info:
      return True
  return False


def _pty_process_name_query(
    address: str, detect_logger: logging.Logger,
    create_switchboard_func: Callable[..., switchboard_base.SwitchboardBase]
) -> str:
  """Returns product name from pty process comms address directory.

  Args:
    address: The communication address.
    detect_logger: The logger of device interactions.
    create_switchboard_func: Method to create the switchboard.
  """
  del create_switchboard_func, detect_logger  # Unused.
  return address


def usb_product_name_query(
    address: str, detect_logger: logging.Logger,
    create_switchboard_func: Callable[..., switchboard_base.SwitchboardBase]
) -> str:
  """Gets product name from usb_info."""
  del create_switchboard_func, detect_logger  # Unused.
  return usb_utils.get_product_name_from_path(address).lower()


def _usb_serial_number_from_serial_port_path(
    address: str, detect_logger: logging.Logger,
    create_switchboard_func: Callable[..., switchboard_base.SwitchboardBase]
) -> str:
  """Gets serial number from serial port path."""
  del create_switchboard_func, detect_logger  # Unused.
  return usb_utils.get_serial_number_from_path(address)


def _usb_vendor_product_id_from_serial_port_path(
    address: str, detect_logger: logging.Logger,
    create_switchboard_func: Callable[..., switchboard_base.SwitchboardBase]
) -> str:
  """Gets USB vendor ID and product ID from serial port path."""
  del create_switchboard_func, detect_logger  # Unused.
  device_info = usb_utils.get_device_info(address)
  return f"{device_info.vendor_id}:{device_info.product_id}"


def _usb_product_name_from_serial_number(
    address: str,
    detect_logger: logging.Logger,
    create_switchboard_func: Callable[..., switchboard_base.SwitchboardBase]
) -> str:
  """Returns USB product name from serial number.

  Args:
    address: Serial number of the device. USB devices are addressed by their
      serial number.
    detect_logger: The logger of device interactions.
    create_switchboard_func: Method to create the switchboard.

  Returns:
    Product name from USB descriptor.
  """
  del create_switchboard_func  # Unused
  device = usb_utils.get_usb_device_from_serial_number(address)
  detect_logger.info(
      "_usb_product_name_from_serial_number: USB device is %r", device)
  return device.product if device else ""


def _usb_vendor_product_id_query(
    address: str, detect_logger: logging.Logger,
    create_switchboard_func: Callable[..., switchboard_base.SwitchboardBase]
) -> str:
  """Returns USB vendor and product ID from serial number.

  Args:
    address: Serial number of the device. USB devices are addressed by their
      serial number.
    detect_logger: The logger of device interactions.
    create_switchboard_func: Method to create the switchboard.

  Returns:
    Vendor and product ID string of the USB device that has a matching
    serial number.
    Format of the string is VENDOR_ID:PRODUCT_ID in hex. E.g. '0ab1:fe23'.
    Returns empty string if address is not found.
  """
  del create_switchboard_func  # Unused by usb_vendor_product_id_query
  # Address contains serial number for USB devices
  device = usb_utils.get_usb_device_from_serial_number(address)
  detect_logger.info("_usb_vendor_product_id_query: USB device is %r", device)
  return f"{device.idVendor:04x}:{device.idProduct:04x}" if device else ""


def _usb_serial_number(
    address: str, detect_logger: logging.Logger,
    create_switchboard_func: Callable[..., switchboard_base.SwitchboardBase]
) -> str:
  """Returns the serial number from a USB device.

  This detection criteria should be used in conjunction with
  _usb_vendor_product_id_query to narrow down a match.

  This function is particularly useful to matching FTDI manufacturer IDs
  which should be consistent between products that use FTDI chipsets. The
  manufacturer's ID is the first two characters in the serial number.
  See:
  https://www.ftdichip.com/Support/Knowledgebase/howistheautomaticserialnu.htm

  Args:
    address: Serial number of the device. USB devices are addressed by their
      serial number.
    detect_logger: The logger of device interactions.
    create_switchboard_func: Method to create the switchboard.

  Returns:
    Serial number.
  """
  del create_switchboard_func, detect_logger  # Unused.
  return address


def _manufacturer_name_query(
    address: str, detect_logger: logging.Logger,
    create_switchboard_func: Callable[..., switchboard_base.SwitchboardBase]
) -> str:
  """Gets manufacturer name from usb_info."""
  del create_switchboard_func, detect_logger  # Unused.
  return usb_utils.get_device_info(address).manufacturer.lower()


def _is_matter_device_query(
    address: str, detect_logger: logging.Logger,
    create_switchboard_func: Callable[..., switchboard_base.SwitchboardBase]
) -> bool:
  """Returns True if the device is a Matter device.

  Args:
    address: The communication address.
    detect_logger: The logger of device interactions.
    create_switchboard_func: Method to create the switchboard.

  Returns:
    True if the device is a Matter device, False otherwise.
  """
  file_handler = typing.cast(logging.FileHandler, detect_logger.handlers[0])
  log_path = file_handler.baseFilename
  return pwrpc_utils.is_matter_device(
      address, log_path, create_switchboard_func, detect_logger)


GENERIC_QUERY_DICT = immutabledict.immutabledict({
    GenericQuery.ALWAYS_TRUE: _always_true_query,
})

ADB_QUERY_DICT = immutabledict.immutabledict({
    AdbQuery.IS_SERIAL_NUMBER: _adb_is_serial_number_query,
})

DOCKER_QUERY_DICT = immutabledict.immutabledict({
    DockerQuery.PRODUCT_NAME: _docker_product_name_query,
})

PIGWEED_QUERY_DICT = immutabledict.immutabledict({
    PigweedQuery.IS_MATTER: _is_matter_device_query,
    PigweedQuery.PRODUCT_NAME: usb_product_name_query,
    PigweedQuery.MANUFACTURER_NAME: _manufacturer_name_query,
})

PTY_PROCESS_QUERY_DICT = immutabledict.immutabledict({
    PtyProcessQuery.PRODUCT_NAME: _pty_process_name_query,
})

SERIAL_QUERY_DICT = immutabledict.immutabledict({
    SerialQuery.PRODUCT_NAME: usb_product_name_query,
    SerialQuery.SERIAL_NUMBER: _usb_serial_number_from_serial_port_path,
    SerialQuery.VENDOR_PRODUCT_ID: _usb_vendor_product_id_from_serial_port_path,
})

SNMP_QUERY_DICT = immutabledict.immutabledict({
    SnmpQuery.IS_DLINK: _is_dlink_query,
})

SSH_QUERY_DICT = immutabledict.immutabledict({
    SshQuery.IS_DLI: _is_dli_query,
    SshQuery.IS_RPI: _is_rpi_query,
    SshQuery.IS_UNIFI: _is_unifi_query,
    SshQuery.IS_CHIP_TOOL_PRESENT: _is_chip_tool_installed_query,
})

USB_QUERY_DICT = immutabledict.immutabledict({
    UsbQuery.PRODUCT_NAME: _usb_product_name_from_serial_number,
    UsbQuery.SERIAL_NUMBER: _usb_serial_number,
    UsbQuery.VENDOR_PRODUCT_ID: _usb_vendor_product_id_query,
})

DETECT_CRITERIA = immutabledict.immutabledict({
    "AdbComms": ADB_QUERY_DICT,
    "DockerComms": DOCKER_QUERY_DICT,
    "JlinkSerialComms": SERIAL_QUERY_DICT,
    "PigweedSerialComms": PIGWEED_QUERY_DICT,
    "PtyProcessComms": PTY_PROCESS_QUERY_DICT,
    "SerialComms": SERIAL_QUERY_DICT,
    "SnmpComms": SNMP_QUERY_DICT,
    "SshComms": SSH_QUERY_DICT,
    "UsbComms": USB_QUERY_DICT,
    "YepkitComms": GENERIC_QUERY_DICT,
})


def determine_device_class(
    address: str, communication_type: str, log_file_path: str,
    create_switchboard_func: Callable[..., switchboard_base.SwitchboardBase]
) -> List[_DeviceClassType]:
  """Returns the device class(es) that matches the address' responses.

  Compares the device_classes DETECT_MATCH_CRITERIA to the device responses.

  Args:
    address: communication_address.
    communication_type: category of communication.
    log_file_path: local path to write log messages to.
    create_switchboard_func: Method to create the switchboard.

  Returns:
    list: classes where the device responses match the detect criteria.
  """
  detect_logger = _setup_logger(log_file_path)
  try:
    device_classes = get_communication_type_classes(communication_type)
    return find_matching_device_class(address, communication_type,
                                      detect_logger, create_switchboard_func,
                                      device_classes)
  finally:
    file_handler = detect_logger.handlers[0]
    file_handler.close()
    detect_logger.removeHandler(file_handler)


def find_matching_device_class(
    address: str, communication_type: str, detect_logger: logging.Logger,
    create_switchboard_func: Callable[..., switchboard_base.SwitchboardBase],
    device_classes: Collection[_DeviceClassType]) -> List[_DeviceClassType]:
  """Returns all classes where the device responses match the detect criteria.

  Args:
    address: communication_address.
    communication_type: category of communication.
    detect_logger: logs device interactions.
    create_switchboard_func: Method to create the switchboard.
    device_classes: device classes whose match criteria must be compared to.

  Returns:
    list: classes where the device responses match the detect criteria.
  """
  matching_classes = []
  responses = _get_detect_query_response(address, communication_type,
                                         detect_logger, create_switchboard_func)
  detect_logger.info(
      "Possible %s device types: %s",
      communication_type,
      [device_class.DEVICE_TYPE for device_class in device_classes])
  for device_class in device_classes:
    if _matches_criteria(responses, device_class.DETECT_MATCH_CRITERIA):
      matching_classes.append(device_class)
      detect_logger.info("\t%s: Match.", device_class.DEVICE_TYPE)
    else:
      detect_logger.info("\t%s: No Match.", device_class.DEVICE_TYPE)
  return matching_classes


def get_communication_type_classes(
    communication_type: str) -> List[_DeviceClassType]:
  """Returns classes with that communication type.

  Args:
    communication_type: category of communication.

  Returns:
    list: classes with that communication type.
  """
  all_classes = copy.copy(extensions.auxiliary_devices)
  all_classes += copy.copy(extensions.primary_devices)
  all_classes += copy.copy(extensions.virtual_devices)
  matching_classes = []
  for device_class in all_classes:
    if device_class.COMMUNICATION_TYPE == communication_type:
      matching_classes.append(device_class)
  return matching_classes


def _get_detect_query_response(
    address: str, communication_type: str, detect_logger: logging.Logger,
    create_switchboard_func: Callable[..., switchboard_base.SwitchboardBase]
) -> Dict[str, Any]:
  """Gathers device responses for all queries of that communication type.

  Args:
    address: communication_address
    communication_type: category of communication.
    detect_logger: logs device interactions.
    create_switchboard_func: Method to create the switchboard.

  Returns:
    Device responses keyed by query enum member.
  """
  query_responses = {}
  detect_queries = extensions.detect_criteria[communication_type]
  for query_name, query in detect_queries.items():
    try:
      query_responses[query_name] = query(
          address=address,
          detect_logger=detect_logger,
          create_switchboard_func=create_switchboard_func)
      detect_logger.info("%s response from %s: %r",
                         query_name, address, query_responses[query_name])
    except Exception as err:  # pylint: disable=broad-except
      detect_logger.info("%s failed for %s: %r", query_name, address, err)
      query_responses[query_name] = repr(err)

    if not isinstance(query_responses[query_name], (str, bool)):
      detect_logger.warning(
          "%s returned invalid response type %s for %s!",
          query_name, type(query_responses[query_name]), address)

  return query_responses


def _matches_criteria(responses, match_criteria):
  """Checks if response dict matches match criteria.

  There are two categories of values in match_criteria: bool and regexp/str.
  Bools must match exactly, while regexp must find a match in the response
  value.

  Args:
    responses (dict): device responses keyed by query name.
    match_criteria (dict): match values keyed by query name.

  Returns:
    bool: whether or not responses meets match criteria
  """
  for entry, value in match_criteria.items():
    if isinstance(value, bool):
      if responses[entry] != value:
        return False
    else:
      if not re.search(value, responses[entry]):
        return False
  return True


def _setup_logger(log_file_path: str) -> logging.Logger:
  """Set up a logger to log device interactions to the detect file."""
  detect_logger = logging.getLogger(log_file_path)
  detect_logger.setLevel(logging.DEBUG)
  handler = logging.FileHandler(log_file_path)
  formatter = logging.Formatter(_LOG_FORMAT)
  handler.setFormatter(formatter)
  detect_logger.addHandler(handler)
  return detect_logger



# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""D-Link switch device controller."""
from typing import Any, Dict, Tuple

from gazoo_device import custom_types
from gazoo_device import decorators
from gazoo_device import detect_criteria
from gazoo_device import errors
from gazoo_device import gdm_logger
from gazoo_device.base_classes import auxiliary_device
from gazoo_device.capabilities import switch_power_snmp
from gazoo_device.utility import host_utils

logger = gdm_logger.get_logger()


class DLinkSwitch(auxiliary_device.AuxiliaryDevice):
  """Device class for a D-Link Switch."""
  COMMUNICATION_TYPE = "SnmpComms"
  DETECT_MATCH_CRITERIA = {detect_criteria.SnmpQuery.IS_DLINK: True}
  DEVICE_TYPE = "dlink_switch"
  _OWNER_EMAIL = "gdm-authors@google.com"

  @decorators.PersistentProperty
  def ip_address(self):
    """Global IP address."""
    return self.communication_address

  @decorators.LogDecorator(logger)
  def get_console_configuration(self) -> None:
    """Returns None. Console is not supported because there's no Switchboard."""
    del self  # Unused because console is not supported.
    return None

  @decorators.CapabilityDecorator(switch_power_snmp.SwitchPowerSnmp)
  def switch_power(self):
    """Capability to control switch power setting over SNMP."""
    return self.lazy_init(
        switch_power_snmp.SwitchPowerSnmp,
        device_name=self.name,
        ip_address=self.ip_address,
        total_ports=self.total_ports)

  @decorators.LogDecorator(logger)
  def get_detection_info(self) -> Tuple[Dict[str, Any], Dict[str, Any]]:
    """Gets the persistent and optional attributes of a DLI Power Switch.

    Returns:
      Tuple of dictionaries containing persistent and optional attributes.
    """
    self.props["optional"] = {}
    self.props["persistent_identifiers"]["model"] = (
        detect_criteria.get_dlink_model_name(self.ip_address))
    # Cannot get serial number via snmp; no other known way for this device.
    # Using IP as a unique identifier instead.
    self.props["persistent_identifiers"]["serial_number"] = (
        self.ip_address.replace(".", ""))
    return self.props["persistent_identifiers"], self.props["optional"]

  @classmethod
  def is_connected(cls,
                   device_config: custom_types.ManagerDeviceConfigDict) -> bool:
    """Checks whether or not the D-Link Power Switch is connected.

    Args:
      device_config: Contains the persistent attributes.

    Returns:
      True if device responds to a ping. False if not.
    """
    return host_utils.is_pingable(
        device_config["persistent"]["console_port_name"])

  @decorators.DynamicProperty
  def total_ports(self) -> int:
    """Gets the number of ports for the attached unifi_switch.

    D-Link DGS-1100 gigabit ethernet switches will have either
    5 or 8 ports depending on their model name.

    Possible model names:
    DGS-1100-05 Gigabit Ethernet Switch
    DGS-1100-05L Gigabit Ethernet Switch
    DGS-1100-08 Gigabit Ethernet Switch
    DGS-1100-08L Gigabit Ethernet Switch

    Returns:
      Number of device ports.
    """
    if "-05" in str(self.model):
      return 5
    elif "-08" in str(self.model):
      return 8
    raise errors.DeviceError(
        f"{self.device_type} model name does not contain expected "
        f"port values -05 or -08. Model name is: {self.model}")



# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Utilities for pty process communications."""
import glob
import os

from gazoo_device import errors


def get_build_path_from_command(command: str) -> str:
  """Returns the directory path from a pty process launch command.

  Args:
    command: The pty process launch command.
  """
  return os.path.dirname(command.split()[0])


def get_launch_command(build_path: str,
                       *,
                       device_image_path_pattern: str,
                       launch_command_template: str,
                       **launch_command_kwargs) -> str:
  """Returns the launch command string from the given command template.

  Args:
    build_path: The path to the simulator build directory.
    device_image_path_pattern: The glob path pattern to device image file,
      can be relative to the build path.
    launch_command_template: The string template to launch command, only
      named formatting values are allowed, e.g. "cmd {device_dir_path}".
    **launch_command_kwargs: Extra keyword values for launch_command_template.
  """
  if not os.path.isabs(device_image_path_pattern):
    device_image_path_pattern = os.path.join(build_path,
                                             device_image_path_pattern)
  device_image_paths = glob.glob(device_image_path_pattern)
  device_image_path = None
  if device_image_paths:
    device_image_path = device_image_paths[0]
  else:
    raise errors.DeviceError("get_launch_command didn't find any simulator "
                             f"images in {device_image_path_pattern}.")
  device_image_dir_path = os.path.dirname(device_image_path)
  if build_path.endswith("/"):
    build_path = build_path[:-1]
  return launch_command_template.format(
      build_dir_path=build_path,
      device_image_dir_path=device_image_dir_path,
      **launch_command_kwargs)



# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Utility module for using the multiprocessing library."""
import multiprocessing
import os

_MP_CONTEXT = multiprocessing.get_context("spawn")


def get_context() -> multiprocessing.context.BaseContext:
  """Returns a multiprocessing context (forkserver- or spawn-based)."""
  return _MP_CONTEXT


def configure_multiprocessing() -> None:
  """Configures multiprocessing. No-op in the open source version."""



# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Interface for a Matter endpoint base capability."""
from typing import Any, Callable, Collection, FrozenSet, List, Optional, Set, Type
from gazoo_device import decorators
from gazoo_device import errors
from gazoo_device import extensions
from gazoo_device import gdm_logger
from gazoo_device.capabilities.interfaces import capability_base
from gazoo_device.capabilities.matter_clusters.interfaces import cluster_base

logger = gdm_logger.get_logger()


class EndpointBase(capability_base.CapabilityBase):
  """Matter endpoint base interface."""

  # Endpoint device type ID defined in the Matter spec.
  DEVICE_TYPE_ID = None

  def __init__(self,
               device_name: str,
               identifier: int,
               supported_clusters: FrozenSet[Type[cluster_base.ClusterBase]],
               read: Callable[..., Any],
               write: Callable[..., Any],
               device_type_id: Optional[int] = None):
    """Initializes Matter endpoint instance.

    Args:
      device_name: Device name used for logging.
      identifier: ID of this endpoint.
      supported_clusters: Supported cluster classes on this endpoint.
      read: Ember API read method.
      write: Ember API write method.
      device_type_id: Device type ID of this endpoint. It's only used for
        unsupported endpoint module, and the supported endpoints use
        cls.DEVICE_TYPE_ID instead.
    """
    super().__init__(device_name=device_name)
    self._id = identifier
    self._device_type_id = device_type_id
    self._supported_clusters = supported_clusters
    self._read = read
    self._write = write

  def __str__(self) -> str:
    """Overrides string representation of the endpoint."""
    return f"{self.name} (endpoint ID: {self.id}) on {self._device_name}"

  @decorators.DynamicProperty
  def id(self) -> int:
    """The ID of the endpoint."""
    return self._id

  @decorators.DynamicProperty
  def name(self) -> str:
    """The name of the endpoint."""
    return self.get_capability_name()

  @decorators.PersistentProperty
  def device_type_id(self) -> int:
    """The device type ID of the endpoint."""
    return self.DEVICE_TYPE_ID

  @decorators.CapabilityLogDecorator(logger)
  def has_clusters(self, cluster_names: Collection[str]) -> bool:
    """Checks whether the endpoint supports all the given cluster names.

    Args:
      cluster_names: The collection of cluster names. The names are
        case-insensitive. Some valid examples are: "color_control",
        "Color_Control".
    Raises:
      ValueError when the given cluster name is invalid or not supported in
      GDM.

    Returns:
      True if the device supports all the clusters, false otherwise.
    """
    valid_cluster_names = {
        cluster_name.replace("_pw_rpc", "")
        for cluster_name, cluster_class in extensions.capability_flavors.items()
        if issubclass(cluster_class, cluster_base.ClusterBase)}

    for cluster_name in cluster_names:
      cluster_name = cluster_name.lower()
      if not cluster_name.endswith("_cluster"):
        cluster_name += "_cluster"
      if cluster_name not in valid_cluster_names:
        raise ValueError(f"Cluster {cluster_name} is not recognized. "
                         f"Supported clusters are {list(valid_cluster_names)}")
      if cluster_name not in self.get_supported_clusters():
        return False
    return True

  @decorators.CapabilityLogDecorator(logger)
  def get_supported_clusters(self) -> List[str]:
    """Returns names of clusters supported by the endpoint."""
    return sorted(
        cluster.get_capability_name() for cluster in self._supported_clusters)

  @decorators.CapabilityLogDecorator(logger)
  def get_supported_cluster_flavors(
      self) -> Set[Type[cluster_base.ClusterBase]]:
    """Returns flavors of cluster supported by the endpoint."""
    return self._supported_clusters

  @decorators.CapabilityLogDecorator(logger, level=decorators.DEBUG)
  def cluster_lazy_init(
      self, cluster_class: Type[cluster_base.ClusterBase]
  ) -> cluster_base.ClusterBase:
    """Provides a lazy instantiation mechanism for Matter cluster.

    Args:
      cluster_class: cluster class to instantiate.

    Returns:
      Initialized Matter cluster instance.
    """
    if cluster_class not in self.get_supported_cluster_flavors():
      raise errors.DeviceError(
          f"{self._device_name} does not support cluster {cluster_class} on "
          f"endpoint {self.name} (endpoint ID {self.id}).")

    cluster_name = cluster_class.__name__
    if not hasattr(self, cluster_name):
      cluster_inst = cluster_class(
          device_name=self._device_name,
          endpoint_id=self.id,
          read=self._read,
          write=self._write)
      setattr(self, cluster_name, cluster_inst)
    return getattr(self, cluster_name)



# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Matter Color Temperature light endpoint.

This endpoint module corresponds to the
"Color Temperature Light" Matter device type which defines required clusters
in the Matter spec 1.0.

A Color Temperature light endpoint houses an instance of Color Temperature Light
Matter device type with the following required clusters on this endpoint:
Scenes, OnOff, Level, Color Control and Groups.
"""
from gazoo_device import decorators
from gazoo_device.capabilities.matter_clusters import color_control_pw_rpc
from gazoo_device.capabilities.matter_clusters import level_control_pw_rpc
from gazoo_device.capabilities.matter_clusters import on_off_pw_rpc
from gazoo_device.capabilities.matter_endpoints.interfaces import color_temperature_light_base


class ColorTemperatureLightEndpoint(
    color_temperature_light_base.ColorTemperatureLightBase):
  """Matter Color Temperature light endpoint."""

  @decorators.CapabilityDecorator(on_off_pw_rpc.OnOffClusterPwRpc)
  def on_off(self) -> on_off_pw_rpc.OnOffClusterPwRpc:
    """Matter OnOff cluster instance."""
    return self.cluster_lazy_init(on_off_pw_rpc.OnOffClusterPwRpc)

  @decorators.CapabilityDecorator(level_control_pw_rpc.LevelControlClusterPwRpc)
  def level(self) -> level_control_pw_rpc.LevelControlClusterPwRpc:
    """Matter Level Control cluster instance."""
    return self.cluster_lazy_init(level_control_pw_rpc.LevelControlClusterPwRpc)

  @decorators.CapabilityDecorator(color_control_pw_rpc.ColorControlClusterPwRpc)
  def color(self) -> color_control_pw_rpc.ColorControlClusterPwRpc:
    """Matter Color Control cluster instance."""
    return self.cluster_lazy_init(color_control_pw_rpc.ColorControlClusterPwRpc)

  # TODO(b/209362086) Add the below clusters
  # def scenes(self):
  #   """Matter Scenes cluster instance."""
  #   # TODO(b/206741536): Implement Scenes cluster.
  #   raise NotImplementedError("Scenes cluster isn't implemented.")

  # def groups(self):
  #   """Matter Groups cluster instance."""
  #   # TODO(b/206741536): Implement Groups cluster.
  #   raise NotImplementedError("Groups cluster isn't implemented.")



# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""SNMP implementation of switch_power."""
import re
import subprocess
from typing import List
from gazoo_device import decorators
from gazoo_device import errors
from gazoo_device import gdm_logger
from gazoo_device.capabilities.interfaces import switch_power_base
from typing_extensions import Literal

logger = gdm_logger.get_logger()

PRIVATE_COMMUNITY = "private"

_GET = "snmpget"
_SET = "snmpset"

_SNMP_TIMEOUT_S = 10

# 1.3.6.1.2.1.2.2.1.7 is the SNMP object identifier (OID) for ifAdminStatus.
# This OID specifies the SNMP endpoint for the status of the switch's ports.
# Querying this OID will list all possible ports and their status.
# ref: https://oidref.com/1.3.6.1.2.1.2.2.1.7
_GET_MODE_SNMP_COMMAND = (
    "{snmp_command} -v 2c -c {community} {ip}:161 1.3.6.1.2.1.2.2.1.7.{port}")
_SET_MODE_SNMP_COMMAND = _GET_MODE_SNMP_COMMAND + " i {mode}"
_MODE_RESPONSE_REG_EX = r".+INTEGER:.+?(\d+)"

# The ifAdminStatus endpoint defines port statuses as 1 or 2.
# The states map like: 1 = ON and 2 = OFF.
_ON = "ON"
_OFF = "OFF"
_GET_MODE_MAPPING = {"1": _ON, "2": _OFF}
_SET_MODE_MAPPING = {_ON: "1", _OFF: "2"}


class SwitchPowerSnmp(switch_power_base.SwitchPowerBase):
  """Switch power flavor for snmp switches."""

  def __init__(self,
               device_name: str,
               ip_address: str,
               total_ports: int,
               community: str = PRIVATE_COMMUNITY):
    """Initializes switch power with SNMP capability.

    Args:
      device_name: Name of the device attached to this capability.
      ip_address: Function to get port status.
      total_ports: Number of network ports present on the device.
      community: Community string configured on switch.
    """
    super().__init__(device_name=device_name)
    self._ip_address = ip_address
    self._total_ports = total_ports
    self._community = community

  @decorators.PersistentProperty
  def supported_modes(self) -> List[str]:
    """Get the power modes supported by the switch."""
    return [_ON, _OFF]

  @decorators.PersistentProperty
  def total_ports(self) -> int:
    """Number of device ports."""
    return int(self._total_ports)

  def get_mode(self, port: int) -> str:
    """Gets the mode for the specified port.

    Args:
      port: Use this port to get the mode.

    Returns:
      Port mode settings "OFF", "ON".

    Raises:
      DeviceError: Raised if passed invalid port.
    """
    self._validate_port(port)
    get_mode_command = _GET_MODE_SNMP_COMMAND.format(
        snmp_command=_GET,
        community=self._community,
        ip=self._ip_address,
        port=port)

    # A command that query's port 1's status looks like:
    # "snmpget -v 2c -c private 123.45.67.89:161 1.3.6.1.2.1.2.2.1.7.1"
    # And its expected response should look like:
    # "iso.3.6.1.2.1.2.2.1.7.1 = INTEGER: 1"
    response = subprocess.check_output(
        get_mode_command.split(), text=True, timeout=_SNMP_TIMEOUT_S)

    match = re.search(_MODE_RESPONSE_REG_EX, response)
    if match and match.group(1) in _GET_MODE_MAPPING.keys():
      return _GET_MODE_MAPPING[match.group(1)]
    raise errors.DeviceError(f"{self._device_name} failed to get port status"
                             f"with command: {get_mode_command}\n"
                             f"Unexpected output: {response}")

  @decorators.CapabilityLogDecorator(logger)
  def set_mode(self, mode: Literal[_ON, _OFF], port: int):
    """Sets the given port to the mode specified.

    Args:
      mode: Mode to set. Options: "OFF", "ON".
      port: The port to set.

    Raises:
      DeviceError: Raised if passed an invalid port or mode.
    """
    self._validate_port(port)
    if mode.upper() not in list(self.supported_modes):
      raise ValueError(f"Attempting to set invalid mode: {mode}. "
                       f"Valid modes are: {self.supported_modes()}.")

    mode = mode.upper()
    set_mode_command = _SET_MODE_SNMP_COMMAND.format(
        snmp_command=_SET,
        community=self._community,
        ip=self._ip_address,
        port=port,
        mode=_SET_MODE_MAPPING[mode])

    # A command that sets port 1's mode to OFF looks like:
    # "snmpset -v 2c -c private 123.45.67.89:161 1.3.6.1.2.1.2.2.1.7.1 i 2"
    # And it's expected response should look like:
    # "iso.3.6.1.2.1.2.2.1.7.1 = INTEGER: 2"
    response = subprocess.check_output(
        set_mode_command.split(), text=True, timeout=_SNMP_TIMEOUT_S)

    match = re.search(_MODE_RESPONSE_REG_EX, response)
    if not match or match.group(1) != _SET_MODE_MAPPING[mode]:
      raise errors.DeviceError(
          f"{self._device_name} Failed to turn {mode.lower()} "
          f"the port using command: '{set_mode_command}'. "
          f"Output: '{response}'")

  @decorators.CapabilityLogDecorator(logger)
  def power_off(self, port: int):
    """This command powers off the port specified.

    Args:
      port: Identifies which hub port to power off.
    """
    self.set_mode(_OFF, port)

  @decorators.CapabilityLogDecorator(logger)
  def power_on(self, port: int):
    """This command powers on the port specified.

    Args:
      port: Identifies which hub port to power on.
    """
    self.set_mode(_ON, port)

  def _validate_port(self, port: int):
    """Ensures port is a valid port number.

    Args:
      port: Device port number.

    Raises:
      DeviceError: if input port is not valid.
    """
    if port <= 0 or port > self._total_ports:
      raise errors.DeviceError(
          f"Port {port} does not exist on {self._device_name}.")



# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Utilities for handling OS signals."""
import contextlib
import dataclasses
import signal
import sys
import types
from typing import NoReturn, Optional

from gazoo_device import gdm_logger

_LOGGER = gdm_logger.get_logger()

# Any unique exit code works here. To help indicate termination by a signal,
# follow the "signal -> exit code" mapping employed by Bash:
# https://tldp.org/LDP/abs/html/exitcodes.html.
_BASH_SIGTERM_EXIT_CODE = 128 + signal.SIGTERM.value


def handle_sigterm(
    signal_number: int, frame: Optional[types.FrameType]) -> NoReturn:
  """Raises a SystemExit exception with _BASH_SIGTERM_EXIT_CODE return code."""
  del signal_number, frame  # Unused.
  sys.exit(_BASH_SIGTERM_EXIT_CODE)


@dataclasses.dataclass
class _SignalReceiver:
  """Receives a signal and stores the signal state: received or not received."""
  received: bool = False

  def handle_signal(
      self, signal_number: int, frame: Optional[types.FrameType]) -> None:
    """Handles a signal."""
    del signal_number, frame  # Unused.
    # Can't log receipt of the signal because logging library calls may not be
    # reentrant: https://docs.python.org/3/library/logging.html#thread-safety.
    self.received = True


@contextlib.contextmanager
def postpone_sigterm_until_completion():
  """Postpones handling of SIGTERM until completion of the context block."""
  signal_receiver = _SignalReceiver()
  previous_sigterm_handler = signal.getsignal(signal.SIGTERM)
  signal.signal(signal.SIGTERM, signal_receiver.handle_signal)
  _LOGGER.info(
      "Postponing SIGTERM handling until completion of the critical section.")
  try:
    yield
  finally:
    _LOGGER.info(
        "Critical section completed. Resuming SIGTERM signal handling.")
    if previous_sigterm_handler is not None:
      signal.signal(signal.SIGTERM, previous_sigterm_handler)
    else:
      signal.signal(signal.SIGTERM, signal.SIG_DFL)
    if signal_receiver.received:
      _LOGGER.info("A SIGTERM was received during the critical section. "
                   f"Exiting with exit code {_BASH_SIGTERM_EXIT_CODE}.")
      sys.exit(_BASH_SIGTERM_EXIT_CODE)



# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Pigweed RPC implementation of the Matter On/Off cluster capability.
"""
from gazoo_device import decorators
from gazoo_device import errors
from gazoo_device import gdm_logger
from gazoo_device.capabilities import matter_enums
from gazoo_device.capabilities.matter_clusters.interfaces import on_off_base
from gazoo_device.protos import attributes_service_pb2

logger = gdm_logger.get_logger()
OnOffCluster = matter_enums.OnOffCluster
BOOLEAN_ATTRIBUTE_TYPE = attributes_service_pb2.AttributeType.ZCL_BOOLEAN_ATTRIBUTE_TYPE


class OnOffClusterPwRpc(on_off_base.OnOffClusterBase):
  """Matter On/Off cluster capability."""

  @decorators.CapabilityLogDecorator(logger)
  def on(self, verify: bool = True) -> None:
    """The On command to turn on the Matter device.

    Args:
      verify: If true, verifies the state change before returning.
    """
    self._onoff_command(on=True, verify=verify)

  @decorators.CapabilityLogDecorator(logger)
  def off(self, verify: bool = True) -> None:
    """The Off command to turn off the Matter device.

    Args:
      verify: If true, verifies the state change before returning.
    """
    self._onoff_command(on=False, verify=verify)

  @decorators.CapabilityLogDecorator(logger)
  def toggle(self, verify: bool = True) -> None:
    """The Toggle command to toggle the OnOff state of the Matter device.

    On receipt of this command, if a device is in its "Off" state it should
    enter its "On" state. Otherwise, if it is in its "On" state it should
    enter its "Off" state.

    Args:
      verify: If true, verifies the state change before returning.
    """
    self._onoff_command(on=not self.onoff, verify=verify)

  @decorators.DynamicProperty
  def onoff(self) -> bool:
    """The OnOff attribute.

    The OnOff attribute indicates the state of the device and has the following
    values: FALSE = "Off" state, TRUE = "On" state.

    Returns:
      True if the device is in on state, false if it's in off state.

    Raises:
      DeviceError: when the ack value is false.
    """
    onoff_data = self._read(
        endpoint_id=self._endpoint_id,
        cluster_id=OnOffCluster.ID,
        attribute_id=OnOffCluster.ATTRIBUTE_ON_OFF,
        attribute_type=BOOLEAN_ATTRIBUTE_TYPE)
    return onoff_data.data_bool

  def _onoff_command(self, on: bool, verify: bool = True) -> None:
    """Turn on/off the device.

    Args:
      on: Turns on the device if true, turn off the device otherwise.
      verify: If true, verifies the device configurations before returning.

    Raises:
      DeviceError: when the ack value if false or the device does not transition
      to the appropriate OnOff state.
    """
    self._write(
        endpoint_id=self._endpoint_id,
        cluster_id=OnOffCluster.ID,
        attribute_id=OnOffCluster.ATTRIBUTE_ON_OFF,
        attribute_type=BOOLEAN_ATTRIBUTE_TYPE,
        data_bool=on)

    if verify:
      if on != self.onoff:  # pylint: disable=comparison-with-callable
        action = "on" if on else "off"
        raise errors.DeviceError(
            f"Device {self._device_name} didn't turn {action}.")



# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Defines Switchboard processes responsible for writing and filtering logs.

The LogFilterProcess is responsible for tailing the log file specified and
producing filter events into the event file specified according to the
following assumptions:

    * The log file will be flushed frequently.

    * If an event file path is not provided then "-events.txt" will be added to
      the log file path provided.

The LogWriterProcess is responsible for receiving log lines added externally
to the provided log_queue and write them to a log file as quickly as possible
according to the following assumptions:

    * Log lines all have a host system timestamp added before being added
      to log queue.

    * Partial log lines contain no newline character at the end

    * Log lines missing a newline character will have one added when written

    * Log lines are queued in the correct order to be written to the log file

"""
import codecs
import datetime
import os
import re
import time

from gazoo_device.switchboard import data_framer
from gazoo_device.switchboard import switchboard_process

CMD_NEW_LOG_FILE = "NEW_LOG_FILE"
CMD_MAX_LOG_SIZE = "MAX_LOG_SIZE"
CMD_ADD_NEW_FILTER = "ADD_NEW_FILTER"
CHANGE_MAX_LOG_SIZE = "Changing max_log_size"
NEW_LOG_FILE_MESSAGE = "Starting new log file at"
ROTATE_LOG_MESSAGE = "Rotating from log file"
LOG_LINE_HEADER_LENGTH = 8  # len(" GDM-#: ")
LOG_LINE_HEADER_FORMAT = r"\sGDM-(.):\s(.*)$"
HOST_TIMESTAMP_LENGTH = 28  # len("<YYYY-MM-DD hh:mm:ss.ssssss>")
HOST_TIMESTAMP_FORMAT = "<%Y-%m-%d %H:%M:%S.%f>"
_MAX_READ_BYTES = 4096
_VALID_COMMON_COMMANDS = [CMD_NEW_LOG_FILE]
_VALID_FILTER_COMMANDS = [CMD_ADD_NEW_FILTER] + _VALID_COMMON_COMMANDS
_VALID_WRITER_COMMANDS = [CMD_MAX_LOG_SIZE] + _VALID_COMMON_COMMANDS


def get_event_filename(log_path):
  """Returns event filename for a given log_path.

  Args:
      log_path (str): path to log filename to get event filename for.

  Returns:
      str: Path to event filename for the given log_path provided.
  """
  return os.path.splitext(log_path)[0] + "-events.txt"


def get_next_log_filename(current_log_path):
  """Returns the next log filename using the current log path as a reference.

  Args:
      current_log_path (str): path to current log filename to get next name
        from.

  Returns:
      str: Path to expected next log filename given current_log_path.

  Note:
      If the current log file name is as follows (the first log file):
          <name_prefix>-<device_name>-<timestamp>.txt

      This method will return the following as the next log filename:
          <name_prefix>-<device_name>-<timestamp>.00001.txt

      Otherwise the rotation count will be extracted and incremented:
          <name_prefix>-<device_name>-<timestamp>.00002.txt
  """
  log_path_no_ext, log_path_ext = os.path.splitext(current_log_path)
  if re.search(r"\.\d{5}$", log_path_no_ext):
    base_log_path, counter_str = log_path_no_ext.rsplit(".", 1)
    counter = int(counter_str)
  else:
    base_log_path = log_path_no_ext
    counter = 0
  next_counter_str = ".{:05}".format(counter + 1)
  return base_log_path + next_counter_str + log_path_ext


def log_message(log_queue, raw_log_line, port):
  """Add host system timestamp to log_line and add result to log_queue.

  Args:
      log_queue (Queue): to send final log message to
      raw_log_line (str): to add system timestamp and GDM log header to
      port (int or str): to identify as source for log line
  """

  switchboard_process.put_message(log_queue,
                                  _add_log_header(raw_log_line, port))


def _add_log_header(raw_log_line, port="M"):
  """Add host system timestamp and GDM log header to raw_log_line.

  Args:
      raw_log_line (str): to add system timestamp and GDM log header to
      port (int or str): to identify as source for GDM log header

  Returns:
      str: The adjusted log line with system timestamp and GDM log header
      added.
  """

  host_timestamp = datetime.datetime.now().strftime(HOST_TIMESTAMP_FORMAT)
  return u"{} GDM-{}: {}".format(host_timestamp, port, raw_log_line)


class LogFilterProcess(switchboard_process.SwitchboardProcess):
  """A process which filters log lines to find device events and records them to an event file.

  The device events are specified as regular expressions in JSON event filter
  files.
  """

  def __init__(self,
               device_name,
               exception_queue,
               command_queue,
               parser,
               log_path,
               max_read_bytes=_MAX_READ_BYTES,
               framer=None):
    """Initialize LogFilterProcess with the arguments provided.

    Args:
        device_name (str): name of device used for process and error
          messages
        exception_queue (Queue): to use for reporting exception traceback
          message from subprocess
        command_queue (Queue): to receive commands into
        parser (Parser): object to use for filtering log lines
        log_path (str): path and filename to write log messages to
        max_read_bytes (int): to attempt to read from log file each time.
        framer (DataFramer): to use to frame log data into partial and
          complete lines.
    """

    super(LogFilterProcess, self).__init__(
        device_name,
        device_name + "-LogFilter",
        exception_queue,
        command_queue,
        valid_commands=_VALID_FILTER_COMMANDS)

    self._buffered_unicode = u""
    self._framer = framer or data_framer.NewlineFramer()
    self._header_length = HOST_TIMESTAMP_LENGTH + LOG_LINE_HEADER_LENGTH
    self._max_read_bytes = max_read_bytes
    self._next_log_path = None
    self._parser = parser
    self._log_file = None
    self._log_filename = os.path.basename(log_path)
    self._log_directory = os.path.dirname(log_path)
    self._event_file = None
    self._event_path = get_event_filename(log_path)

  def _close_files(self):
    if hasattr(self, "_event_file") and self._event_file:
      self._event_file.flush()
      self._event_file.close()
      self._event_file = None
    if hasattr(self, "_log_file") and self._log_file:
      self._log_file.close()
      self._log_file = None

  def _is_log_swap_or_rotation(self, log_line):
    """Returns True if log_line indicates a log swap or rotation should occur.

    Args:
        log_line (str): to check for log swap indicator

    Returns:
        bool: True if log swap or rotation should occur.

    Note:
        If the log_line contains the ROTATE_LOG_MESSAGE then this method
        will identify what the next log file should be using the current
        log path and append it to the list of next log files to switch to.

        If the log_line contains the NEW_LOG_FILE_MESSAGE then the next log
        file was already prepended to the list of next log files when the
        command was received.

        It is expected that the LogWriterProcess will only allow one or the
        other message to be written to any given log file and not both.
    """

    result = False
    if ROTATE_LOG_MESSAGE in log_line:
      result = True
      log_path = os.path.join(self._log_directory, self._log_filename)
      next_log_path = get_next_log_filename(log_path)
      self._next_log_path.append(next_log_path)
    elif NEW_LOG_FILE_MESSAGE in log_line and self._next_log_path:
      self._event_path = get_event_filename(self._next_log_path[-1])
      result = True
    return result

  def _do_work(self):
    """Perform log filtering work.

    Returns:
        bool: indicating process should continue running

    Note:
        This implementation always returns True
    """

    command_message = switchboard_process.get_message(
        self._command_queue, timeout=0)
    if command_message:
      self._process_command_message(command_message)
    if self._has_log_file(self._log_filename):
      if not self._is_open():
        self._open_log_file(self._log_filename)
        if hasattr(self, "_log_file") and self._log_file:
          self._open_event_file()
      self._filter_log_lines()
    else:
      time.sleep(0.01)

    return True

  def _has_log_file(self, log_filename):
    log_path = os.path.join(self._log_directory, log_filename)
    return os.path.isfile(log_path)

  def _is_open(self):
    return (hasattr(self, "_log_file") and self._log_file and
            hasattr(self, "_event_file") and self._event_file)

  def _open_event_file(self):
    self._event_file = codecs.open(self._event_path, "a", encoding="utf-8")

  def _open_log_file(self, log_filename):
    log_path = os.path.join(self._log_directory, log_filename)
    self._log_file = codecs.open(
        log_path, "r", encoding="utf-8", errors="replace")
    self._log_filename = log_filename

  def _filter_log_lines(self):
    # Can't use readline here or splitlines later below because some devices
    # emit multiple return characters on each
    # log line read which prevents readline and splitlines from working
    # correctly. Each log line is expected to only have 1 newline but could
    # have multiple line return characters (see NEP-2855).
    change_log_file = False

    # NEP-3848: in Python 2, for file descriptor Foo whose EOF flag is set
    # and another file descriptor Bar which then appends to the same file,
    # the next read() for file descriptor Foo will return "" even though B has added new data.
    self._log_file.seek(self._log_file.tell())  # Unset EOF flag
    log_data = self._log_file.read(size=self._max_read_bytes)

    if log_data:
      log_lines = self._buffered_unicode + log_data
      buffered_len = len(self._buffered_unicode)
      self._buffered_unicode = u""
      for log_line in self._framer.get_lines(log_lines, begin=buffered_len):
        if log_line[-1] == "\n":
          self._parser.process_line(
              self._event_file,
              log_line,
              header_length=self._header_length,
              log_filename=self._log_filename)
        else:
          self._buffered_unicode += log_line
        if self._is_log_swap_or_rotation(log_line):
          change_log_file = True
    else:
      time.sleep(0.01)
    if change_log_file:
      self._open_next_log_file()

  def _open_next_log_file(self):
    try:
      new_log_path = self._next_log_path.pop()
    except IndexError:
      # No new log file
      pass
    else:
      self._close_files()
      self._buffered_unicode = u""
      self._log_directory = os.path.dirname(new_log_path)
      self._log_filename = os.path.basename(new_log_path)

  def _post_run_hook(self):
    self._close_files()

  def _pre_run_hook(self):
    self._next_log_path = []
    return True

  def _process_command_message(self, command_message):
    """Processes command messages which are received as a (command, data) tuple.

    Args:
        command_message (tuple): as received from command queue

    Raises:
        RuntimeError: if command in command_message is unknown.
    """

    command, data = command_message
    if CMD_NEW_LOG_FILE == command:
      self._next_log_path.insert(0, data)
    elif CMD_ADD_NEW_FILTER == command:
      self._parser.load_filter_file(data)
    else:
      raise RuntimeError("Device {} received an unknown command {}.".format(
          self.device_name, command))


class LogWriterProcess(switchboard_process.SwitchboardProcess):
  """A process which writes all logs found in the log queue into a log file.

  It expects each log line to be prepended with a host system timestamp.
  Log lines that are missing a newline character will have one added.
  Partial log lines should be handled by log line producers.
  """

  def __init__(self,
               device_name,
               exception_queue,
               command_queue,
               log_queue,
               log_path,
               max_log_size=0):
    """Initialize LogWriterProcess with the arguments provided.

    Args:
        device_name (str): name of device used for process and error
          messages
        exception_queue (Queue): to use for reporting exception traceback
          message from subprocess
        command_queue (Queue): to receive commands into
        log_queue (Queue): to retrieve log messages from to be written
        log_path (str): path and filename to write log messages to
        max_log_size (int): maximum size in bytes before performing log
          rotation

    Note: A max_log_size of 0 means no log rotation should ever occur.
    """

    super(LogWriterProcess, self).__init__(
        device_name,
        device_name + "-LogWriter",
        exception_queue,
        command_queue,
        valid_commands=_VALID_WRITER_COMMANDS)
    self._log_queue = log_queue
    self._log_filename = os.path.basename(log_path)
    self._log_directory = os.path.dirname(log_path)
    self._log_file = None
    self._max_log_size = max_log_size

  def _close_file(self):
    if hasattr(self, "_log_file") and self._log_file:
      self._log_file.flush()
      self._log_file.close()

  def _do_log_rotation(self):
    """Perform log rotation if necessary."""
    if self._max_log_size:
      log_size = self._log_file.tell()
      if log_size >= self._max_log_size:
        new_log_filename = get_next_log_filename(self._log_filename)
        raw_log_message = "{} {} to {}\n".format(ROTATE_LOG_MESSAGE,
                                                 self._log_filename,
                                                 new_log_filename)
        self._write_log_line(_add_log_header(raw_log_message))
        new_log_path = os.path.join(self._log_directory, new_log_filename)
        self._open_new_log_file(new_log_path)

  def _do_work(self):
    """Perform log writing work.

    Returns:
        bool: indicating process should continue running

    Note:
        This implementation always returns True
    """
    command_message = switchboard_process.get_message(
        self._command_queue, timeout=0)
    if command_message:
      self._process_command_message(command_message)
    log_line = switchboard_process.get_message(self._log_queue, timeout=0.01)
    if log_line:
      self._write_log_line(log_line)
      self._do_log_rotation()
    else:
      time.sleep(0.01)

    return True

  def _open_file(self):
    if self._log_directory and not os.path.exists(self._log_directory):
      os.makedirs(self._log_directory)
    log_path = os.path.join(self._log_directory, self._log_filename)
    self._log_file = codecs.open(log_path, "a", encoding="utf-8")

  def _open_new_log_file(self, new_log_path):
    self._close_file()
    self._log_directory = os.path.dirname(new_log_path)
    self._log_filename = os.path.basename(new_log_path)
    self._open_file()

  def _post_run_hook(self):
    self._close_file()

  def _pre_run_hook(self):
    self._open_file()
    return hasattr(self, "_log_file") and self._log_file

  def _process_command_message(self, command_message):
    """Processes command messages which are received as a (command, data) tuple.

    Args:
        command_message (tuple): as received from command queue

    Raises:
        RuntimeError: if command in command_message is unknown.
    """

    command, data = command_message
    if CMD_MAX_LOG_SIZE == command:
      raw_log_message = "{} from {} to {}\n".format(CHANGE_MAX_LOG_SIZE,
                                                    self._max_log_size, data)
      self._write_log_line(_add_log_header(raw_log_message))
      self._max_log_size = data
    elif CMD_NEW_LOG_FILE == command:
      raw_log_message = "{} {}\n".format(NEW_LOG_FILE_MESSAGE, data)
      self._write_log_line(_add_log_header(raw_log_message))
      self._open_new_log_file(data)
    else:
      raise RuntimeError("Device {} received an unknown command {}.".format(
          self.device_name, command))

  def _write_log_line(self, log_line):
    if hasattr(self, "_log_file") and self._log_file:
      if log_line[-1] != "\n":
        # Write log line with newline added
        self._log_file.write(log_line + "[NO EOL]\n")
      else:
        self._log_file.write(log_line)
      self._log_file.flush()



# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Module for log parser."""
import codecs
import os
import time

from gazoo_device import errors
from gazoo_device import gdm_logger
from gazoo_device.switchboard import data_framer
from gazoo_device.switchboard import log_process

logger = gdm_logger.get_logger("log_parser")

DISPLAY_REFRESH = 3.0  # secs


class LogParser(object):
  """Provides ability to search for specific events in log file."""

  def __init__(self, parser_obj, log_path, display_refresh=DISPLAY_REFRESH):
    """Initialize LogParser class using provided information.

    Args:
        parser_obj (Parser): Instance of class Parser
        log_path (str): Path to log filename containing raw, log event data
        display_refresh (float): Number of seconds to wait prior to refresh
          of display

    Raises:
        ParserError: If log_path does NOT exist
                     If event_filename already exists
                     If parser_object is None

    Note:
         Since the provided log_path is immediately parsed, initializing
    LogParser using log files exceeding 100 MB can cause test applications
    to appear to be delayed.  The smaller the log file the faster
    initialization will be.
    """

    if parser_obj is None:
      raise errors.ParserError("Log parser parameter check failed. "
                               "Bad parser_obj.")

    if not os.path.isfile(log_path):
      raise errors.ParserError(
          "LogParser parameter check failed. "
          "log file name: {} does not exist.".format(log_path))

    self.event_filename = log_process.get_event_filename(log_path)
    parser_obj.event_file_path = self.event_filename
    if os.path.isfile(self.event_filename):
      raise errors.ParserError("LogParser parameter check failed. "
                               "event_filename: {} already exists.".format(
                                   self.event_filename))
    if display_refresh < 0:
      raise errors.ParserError(
          "LogParser parameter check failed. "
          "Expected display refresh >=0 instead got: {}".format(
              display_refresh))
    self._parser_obj = parser_obj
    self._parse_events(log_path, display_refresh)

  def get_last_event(self, event_labels=None, timeout=1.0):
    r"""Returns the most recent matching event for each item in the list of event labels.

    Args:
        event_labels (list): list of event labels to lookup (example:
          ["power.state", "power.lost"]). None (default) returns the most
          recent event regardless of label.
        timeout (float): Timeout value in seconds. Example: 1.5.

    Raises:
        ParserError: on error parsing event file.

    Returns:
        ParserResult: An object containing the most recent event match for each
        event label in the event_labels list, or the most recent event if
        event_labels is None. The event data is stored in the .results_list
        attribute, and the count of results is in the .count attribute. If no
        matching events are found, the .results_list attribute will be set to
        the empty list.

        If the .timedout attribute is True, the operation timed out before all
        events were searched or the event file was not found before the timeout
        expired.

    Notes:
        The event_label is the desired filter filename without extension a .
        and the filter name (e.g. power.state). The following examples use a
        hypothetical power.json filter file with the following filters:

        .. code-block:: none

            {
                "name": "state",
                "regex_match": "power:(\\w+)"
            },
            {
                "name": "lost",
                "regex_match": "power lost"
            },
            {
                "name": "restored",
                "regex_match": "power restored"
            }

        The original log file being read looks like this:

        .. code-block:: none

            <2018-02-02 10:32:07.167234> [APPL] Spoke: power restored,
            powering on
            <2018-02-02 12:00:57.154328> [APPL] Spoke: power lost, powering
            down
            <2018-02-02 12:00:57.154328> power:OFF
            <2018-02-02 12:01:22.122824> [APPL] Spoke: power restored,
            powering on
            <2018-02-02 12:01:22.122824> power:ON
            <2018-02-02 12:02:18.121948> [APPL] Spoke: power lost, powering
            down
            <2018-02-02 12:02:18.121948> power:OFF

        Example output with event_labels=["power.state"], showing group
        matches:

        .. code-block:: none

            result.timedout = False
            result.results_list =
                [{'system_timestamp': datetime.datetime(2018, 2, 2, 12, 2,
                18, 121948),
                  'raw_log_line': 'power:OFF',
                  'power.state': ['OFF']}]
            result.count = 1

        Example output with event_labels=["power.lost"], showing matches
        with no groups:

        .. code-block:: none

            result.timedout = False
            result.results_list =
                [{'system_timestamp': datetime.datetime(2018, 2, 2, 12, 2,
                18, 121948),
                  'raw_log_line': '[APPL] Spoke: power lost',
                  'power.lost': []}]
            result.count = 1

        Example output with event_labels=["power.state", "power.lost",
        "power.restored"]:

        .. code-block:: none

            result.timedout = False
            result.results_list =
                [{'system_timestamp': datetime(2018, 2, 2, 12, 2, 18,
                121948),
                  'raw_log_line': 'power:OFF',
                  'power.state': ['OFF']},
                 {'system_timestamp': datetime(2018, 2, 2, 12, 2, 18,
                 121948),
                  'raw_log_line': '[APPL] Spoke: power lost',
                  'power.lost': []}
                 {'system_timestamp': datetime(2018, 2, 2, 12, 1, 22,
                 122824),
                  'raw_log_line': '[APPL] Spoke: power restored, powering
                  on',
                  'power.restored': []}]
            results.count = 3
    """
    if event_labels:
      if not isinstance(event_labels, list):
        raise errors.ParserError("Expecting event_labels to be a list "
                                 "or None but received {} instead.".format(
                                     type(event_labels)))
      cur_event_labels = self.get_event_labels()
      for event_label in event_labels:
        if event_label not in cur_event_labels:
          raise errors.ParserError(
              "Event label {} doesn't exist.".format(event_label))

    return self._parser_obj.get_last_event(
        self.event_filename, event_labels, timeout=timeout)

  def get_event_history(self, event_labels=None, count=None, timeout=10.0):
    r"""Returns up to count elements of event data matching given event labels.

    Args:
        event_labels (list): list of event labels to lookup (example:
          ["power.state", "power.lost"]). If None (default), return all
          events.
        count (int): of event data elements to return (Default: None for all
          events).
        timeout (float): Timeout value in seconds. Example: 1.5.

    Raises:
        ParserError: event_labels is not type list or
                     label in event_labels doesn't exist or
                     on error parsing event file

    Returns:
        ParserResult: An object containing events matching any of the event
        labels provided in event_labels, or all available events if event_labels
        in None. The event data is stored in the .results_list attribute and the
        count of results in the .count attribute. If no matching events are
        found, the .results_list attribute will be set to the empty list.

    If the .timedout attribute is True, the operation timed out before all
    events were searched, or the event file was not found before the timeout
    expired.

    Notes:
        Event info is listed in most recent to least recent order.

        The following examples use a hypothetical power.json filter file
        with the
        following filters:

        .. code-block:: none

            {
                "name": "state",
                "regex_match": "power:(\\w+)"
            },
            {
                "name": "lost",
                "regex_match": "power lost"
            },
            {
                "name": "restored",
                "regex_match": "power restored"
            }

        The original log file being read looks like this:

        .. code-block:: none

            <2018-02-02 10:32:07.167234> [APPL] Spoke: power restored,
            powering on
            <2018-02-02 12:00:57.154328> [APPL] Spoke: power lost, powering
            down
            <2018-02-02 12:00:57.154328> power:OFF
            <2018-02-02 12:01:22.122824> [APPL] Spoke: power restored,
            powering on
            <2018-02-02 12:01:22.122824> power:ON
            <2018-02-02 12:02:18.121948> [APPL] Spoke: power lost, powering
            down
            <2018-02-02 12:02:18.121948> power:OFF

        Example output with event_labels ["power.state"], showing group
        matches:

        .. code-block:: none

            result.timedout = False
            result.results_list =
                [{'system_timestamp': datetime.datetime(2018, 2, 2, 12, 2,
                18, 121948),
                  'raw_log_line': 'power:OFF',
                  'power.state': ['OFF']},
                 {'system_timestamp': datetime.datetime(2018, 2, 2, 12, 1,
                 22, 122824),
                  'raw_log_line': 'power:ON',
                  'power.state': ['ON']},
                 {'system_timestamp': datetime.datetime(2018, 2, 2, 12, 0,
                 57, 154328),
                  'raw_log_line': 'power:OFF',
                  'power.state': ['OFF']}]
            result.count = 3

        Example output with event_labels ["power.lost"], showing matches
        with no groups:

        .. code-block:: none

            result.timedout = False
            result.results_list =
                [{'system_timestamp': datetime.datetime(2018, 2, 2, 12, 2,
                18, 121948),
                  'raw_log_line: '[APPL] Spoke: power lost, powering down',
                  'power.lost': []},
                 {'system_timestamp': datetime.datetime(2018, 2, 2, 12, 0,
                 57, 154328),
                  'raw_log_line': '[APPL] Spoke: power lost, powering down',
                  'power.lost': []}]
            result.count = 2

        Example output with event_labels ["power.restored"], showing matches
        with no
        groups and count=1:

        .. code-block:: none

            result.timedout = False
            result.results_list =
                [{'system_timestamp': datetime.datetime(2018, 2, 2, 12, 1,
                22, 122824),
                  'raw_log_line': '[APPL] Spoke: power restored, powering
                  on',
                  'power.restored': []}
            result.count = 1

        Example output with event_labels ["power.lost", "power.restored"]:

        .. code-block:: none

            result.timedout = False
            result.results_list =
                [{'system_timestamp': datetime.datetime(2018, 2, 2, 12, 2,
                18, 121948),
                  'raw_log_line: '[APPL] Spoke: power lost, powering down',
                  'power.lost': []},
                 {'system_timestamp': datetime.datetime(2018, 2, 2, 12, 1,
                 22, 122824),
                  'raw_log_line': '[APPL] Spoke: power restored, powering
                  on',
                  'power.restored': []},
                 {'system_timestamp': datetime.datetime(2018, 2, 2, 12, 0,
                 57, 154328),
                  'raw_log_line': '[APPL] Spoke: power lost, powering down',
                  'power.lost': []}
                 {'system_timestamp': datetime.datetime(2018, 2, 2, 10, 32,
                 7, 167234),
                  'raw_log_line': '[APPL] Spoke: power restored, powering
                  on',
                  'power.restored': []}]
             result.count = 4
    """
    if event_labels:
      if not isinstance(event_labels, list):
        raise errors.ParserError("Expecting event_labels to be a list "
                                 "or None but received {} instead.".format(
                                     type(event_labels)))
      cur_event_labels = self.get_event_labels()
      for event_label in event_labels:
        if event_label not in cur_event_labels:
          raise errors.ParserError(
              "Event label {} doesn't exist.".format(event_label))

    return self._parser_obj.get_event_history(
        event_labels, count=count, timeout=timeout)

  def get_event_history_count(self, event_label, timeout=10.0):
    """Returns count of event history matching event label specified.

    Args:
        event_label (str): event label to lookup (e.g. power.state)
        timeout (float): Timeout value in seconds. Example: 1.5.

    Raises:
        ParserError: on error parsing event file or event_label doesn't exist

    Returns:
        ParserResult: An object containing the count of results in the .count
        attribute. The .results_list field is set to the empty list.

        If the .timedout attribute is True, the operation timed out before all
        events were searched, or the event file was not found before the timeout
        expired.
    """
    if event_label not in self.get_event_labels():
      raise errors.ParserError(
          "Event label {} doesn't exist.".format(event_label))

    return self._parser_obj.get_event_history_count(
        self.event_filename, event_label, timeout=timeout)

  def get_event_label_dict(self, pattern=None):
    """Takes an event label pattern, and returns a dictionary of matching event labels.

    Args:
        pattern (str): An event label pattern regular expression

    Returns:
        dict: containing keys (i.e. event labels), and values (i.e regular
        expressions)

    Raises:
        ParserError: if pattern provided is an invalid regular expression

    Note:
        The pattern regular expression provided will be matched strictly
        against the beginning of each event_label.
        If no pattern is provided, the _filters_dict is returned.
    """
    return self._parser_obj.get_event_label_dict(pattern=pattern)

  def get_event_labels(self, pattern=None):
    """Takes an event label pattern, and returns a list of matching event labels.

    Args:
        pattern (str): An event label pattern regular expression

    Returns:
        list: of matching event label strings

    Raises:
        ParserError: if pattern provided is an invalid regular expression

    Note:
        The pattern regular expression provided will be matched strictly
        against the beginning of each event_label.
        If no pattern is provided, all filter event labels are returned.
    """
    return self._parser_obj.get_event_labels(pattern=pattern)

  def _parse_events(self, log_path, display_refresh):
    """Parses log file searching for events depositing results into an event file.

    Args:
        log_path (str): Path to log filename containing raw, log event data
        display_refresh (float): Number of seconds to wait prior to refresh
          of display

    Raises:
        ParserError: if log parser fails.

    Note:
        With large log files, this process may take a large amount of time.
    """
    logger.info("Parsing log file %s into event file %s, please wait", log_path,
                self.event_filename)
    start_time = time.time()
    with codecs.open(self.event_filename, "a", encoding="utf-8") as event_file:
      log_filename = os.path.basename(log_path)
      with codecs.open(
          log_path, "r", encoding="utf-8", errors="replace") as log_file:
        log_file.seek(0, os.SEEK_END)
        total_bytes = log_file.tell()
        log_file.seek(0, os.SEEK_SET)
        process_time = start_time
        buffered_unicode = u""
        framer = data_framer.NewlineFramer()
        new_header_length = (
            log_process.HOST_TIMESTAMP_LENGTH +
            log_process.LOG_LINE_HEADER_LENGTH)
        old_header_length = 29
        try:
          while True:
            log_data = log_file.read(size=4096)
            if time.time() - process_time > display_refresh:
              process_time = time.time()
              bytes_processed = log_file.tell()
              logger.info("%.2f%% complete - bytes processed: %d of %d",
                          100 * bytes_processed / total_bytes, bytes_processed,
                          total_bytes)
            if not log_data:
              break

            log_lines = buffered_unicode + log_data
            buffered_len = len(buffered_unicode)
            buffered_unicode = u""
            for log_line in framer.get_lines(log_lines, begin=buffered_len):
              if log_line[-1] == "\n":
                if "> GDM-" in log_line:
                  header_length = new_header_length
                else:
                  header_length = old_header_length
                self._parser_obj.process_line(
                    event_file,
                    log_line,
                    header_length=header_length,
                    log_filename=log_filename)
              else:
                buffered_unicode += log_line
        except IOError as err:
          logger.debug("log_parser encountered error: {!r}".format(err))
          raise errors.ParserError("Log file processing failed. "
                                   "IOError: {!r}".format(err))
    logger.info("Parsing log file %s into event file %s finished in %ds",
                log_path, self.event_filename,
                time.time() - start_time)



# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Utilities for adding an alias that will log a message and call a method or return a property."""
# Deprecation notice for Sphinx documentation
SPHINX_DEPRECATION_NOTICE = ('This {attr_type} has been deprecated. See '
                             '"{new_name}".')


def _get_nested_attr(obj, name):
  """Get an attribute that may or may not be nested within an object.

  This will get an attribute of obj or or an attribute that is in an object
  that is in obj.

  Args:
      obj (object): The object to check for the attribute with the given name.
      name (str): The name of the attribute to get. name can be either an
        attribute of obj or an object.attribute where object is part of obj.

  Returns:
      object: The method or property that may have been nested inside of other
      objects.
  """
  attr_list = name.split('.')
  for attr in attr_list:
    obj = getattr(obj, attr, None)
  return obj


def add_deprecated_attribute(cls, old_name, new_name, is_method):
  """Add a single deprecated property or method to a class.

  The "old_name" that is provided will be
  added to the provided class as an alias to the "new_name". Then each time
  the attribute with
  "old_name" is used it will log a deprecated message and return as if the
  "new_name" attribute
  was called.

  Note: This must be called for a class and not for an instance of a class
  since dynamically
        adding a property cannot be done for an instance of a class.

  Args:
      cls (class): The class to add the attribute to.
      old_name (str): The name of the old attribute that will be added as an
        alias for "new_name".
      new_name (str): The new of the new attribute.
      is_method (bool): True if the attribute is a method, or False for a
        property.
  """

  def wrapper(obj, *args, **kwargs):
    """A wrapper function that is called for the added attribute.

    Args:
        obj (object): The object that contains the attribute.
        *args: The required arguments for the method.
        **kwargs: The keyword arguments for the method.

    Returns:
        value: The property or what was returned from the method.
    """
    # Log a deprecated message here after we have announced capabilities are ready.
    attr = _get_nested_attr(obj, new_name)
    if is_method:
      return attr(*args, **kwargs)
    else:
      return attr

  # mark deprecated for documentation
  wrapper.__deprecated__ = new_name
  wrapper.__doc__ = SPHINX_DEPRECATION_NOTICE.format(
      attr_type='method' if is_method else 'property', new_name=new_name)

  if is_method:
    setattr(cls, old_name, wrapper)
  else:
    setattr(cls, old_name, property(wrapper))


def add_deprecated_attributes(cls, attribute_list):
  """Add each attribute in a list of tuples to the class provided.

  Note: This must be called for a class and not for an instance of a class
  since dynamically adding a property cannot be done for an instance of a class.

  Args:
      cls (class): The class to add the attribute to.
      attribute_list (list): A list of tuples with the attributes to add where
        the tuple has: old_name, new_name, is_method
  """
  for old_name, new_name, is_method in attribute_list:
    add_deprecated_attribute(cls, old_name, new_name, is_method)
